#!/usr/bin/perl

use strict;
use FDN::Common::Logger;
use FDN::Sql::Workflow::Run;
use FDN::Adsl::Basic;

workflow_select_db('adsl');
workflow_select_config('adsl/wf.cfg');

sub x_error($);
sub traite_fichier($) {
	my ( $filename ) = @_;
	if ( $filename =~ /\.agp\.arfic$/ ) {
		traite_fichier_arfic($filename);
	} elsif ( $filename =~ /\.agp\.rej$/ ) {
		traite_fichier_rejet($filename);
	} elsif ( $filename =~ /\.agp\.ar$/ ) {
		traite_fichier_ar($filename);
	} elsif ( $filename =~ /\.agp\.cr$/ ) {
		traite_fichier_cr($filename);
	} elsif ( $filename =~ /\.no$/ ) {
		traite_fichier_notif($filename);
	} else {
		x_error "Fichier innatendu: $filename";
	}
}

sub traite_fichier_arfic($) {
	my ( $filename ) = @_;

	if ( ! open IN,"<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $line = <IN>;
	close IN;

	$line =~ /^(\d{8});([^;]*);(\d*)/;
	my ( $date, $file, $lignes ) = ( $1, $2, $3 );
	$file =~ s/\.arfic$//;
	
	my @ordres = ordretd_par_fichier($file);
	if ( array_has_no_rows(\@ordres) ) {
		x_error "Recu un fichier ARFIC: $filename qui ne correspond par a des ordres";
		return;
	}
	if ( $#ordres+2 != $lignes ) {
		x_error "J'ai envoye ".($#ordres+2)." lignes dont une d'en-tete. TD dit n'en avoir recu que $lignes";
	}
	# Lire le fichier, tout de meme!
	for my $o ( @ordres ) {
		my $oid = $o->{'ORDRETD_ID'};
		send_event('ARFIC-'.$oid);
	}
}

my $rejet_fichier = {
	'E0'  => "Mode d'echange incompatible avec le contrat",
	'E1'  => "Entete du fichier pas au bon format",
	'E2'  => "Valeur du champ operateur inconnu",
	'E3'  => "Erreur dans l'entete du fichier: format de date invalide",
	'E4'  => "Erreur dans l'entete du fichier: domaine inconnu",
	'E5'  => "Nom de fichier invalide",
	'E6'  => "Erreur dans le nom du fichier: date au format invalide",
	'E7'  => "Erreur dans le nom du fichier: code operateur invalide",
	'E8'  => "Erreur dans le nom du fichier: numero de sequence au format invalide",
	'E9'  => "Erreur dans le nom du fichier: suffixe invalide",
	'E10' => "Nombre de lignes maximal depasse",
	'E11' => "Code operateur different entre nom du fichier et en-tete",
	'E12' => "Domaine incompatible avec le type de fichier",
	'E13' => "Nombre de champs invalide dans une ligne du fichier",
	'E14' => "Code operateur inconnu (repertoire de reception non identifie)",
	'E15' => "Fichier deja recu",
	'E16' => "Impossible de decrypter le fichier",
	'E17' => "Un champ comporte plus de 512 caracteres",
	'E18' => "Protocole inconnu"
};

sub traite_fichier_rejet($) {
	my ( $filename ) = @_;
	x_error "Un fichier a ete rejete par TD";

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne = <IN>;
	close IN;

	$ligne =~ /^(\d{8});([^;]*);([A-Z]\d+)/;
	my ( $date, $file, $err ) = ( $1, $2, $3 );
	
	if ( ! defined $file ) {
		x_error "Pas russi  comprendre le nom du fichier... J'arrete les frais.";
		return;
	}
	x_error "Fichier en erreur: $file";
	
	if ( ! defined $rejet_fichier->{$err} ) {
		x_error "Code d'erreur <$err> inconnu au bataillon. J'arrete l.";
		return;
	}
	x_error "Code d'erreur: ".$err." (".$rejet_fichier->{$err}.")";
	
	if ( $err eq 'E15' ) {
		x_error "TD dit avoir deja recu le fichier, je ne change rien!";
	} else {
		x_error "L'erreur est plus grave, je marque les ordres en attente.";
		my @ordres = ordretd_par_fichier($file);
		if ( array_has_no_rows(\@ordres) ) {
			x_error "Aucun ordre ne correspond a ce fichier en erreur!";
			return;
		}
		for my $o ( @ordres ) {
			my $oid = $o->{'ORDRETD_ID'};
			if ( ! $oid ) {
				x_error "Curieux: un ordre attach au fichier n'a pas d'identifiant valide... Suivant.";
				next;
			}
			x_error "Je retabli l'ordre numero $oid qui disait:";
			x_error $o->{'ORDRETD_LIGNE'};
			set_ordretd_etat($oid,'new');
			set_ordretd_fichier($oid,'');
		}
	}
}

sub traite_fichier_ar($) {
	my ( $filename ) = @_;

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne;
	my $head = <IN>;
	while ( $ligne = <IN> ) {
		chomp $ligne;
		$ligne =~ s/\r$//;
		my @infos = split /;/, $ligne;
		# Les infos nouvelles
		my $recevabilite = $infos[0];
		my $numero_commande = $infos[33];
		my $date_recu = $infos[34];
		my $date_traitee = $infos[35];
		my $code_rejet = $infos[36];
		my $commentaire = $infos[37];
		# L'ordre de depart
		my @ordre = @infos[1..32];
		my $ligne_orig = join ';', @ordre;
		my $oid = $ordre[2];
		my $telephone = $ordre[11];
		my $ordre = ordretd_par_id($oid);
		if ( $ligne_orig ne $ordre->{'ORDRETD_LIGNE'} ) {
			x_error "La reponse ne colle pas absolument avec la question.";
			x_error "La question etait:";
			x_error $ordre->{'ORDRETD_LIGNE'};
			x_error "La reponse porte sur:";
			x_error $ligne_orig;
		}
		my $event_ar = 'ARTD-'.$oid;
		my $event_props = {
			'oid' => $oid,
			'telephone' => $telephone,
			'statut' => $recevabilite,
			'numcde' => $numero_commande,
			'date-recu' => $date_recu,
			'date-traitee' => $date_traitee,
			'code-rejet' => $code_rejet,
			'commentaire' => $commentaire
			};
		if ( $recevabilite eq 'TRANSMISE' ) {
			set_ordretd_etat($oid,'ok');
		} elsif ( $recevabilite eq 'REJETEE' ) {
			set_ordretd_etat($oid,'ko');
		} else {
			x_error "Le status de l'AR est incomprehensible: $recevabilite";
			x_error "Dans le fichier: $filename";
			x_error "La ligne:";
			x_error $ligne;
			return;
		}
		send_event($event_ar,$event_props);
	}
}

sub traite_fichier_cr($) {
	my ( $filename ) = @_;

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne;
	my $head = <IN>;
	while ( $ligne = <IN> ) {
		chomp $ligne;
		$ligne =~ s/\r$//;
		my @infos = split /;/, $ligne;
		# Les infos nouvelles
		my $etat = $infos[0];
		my $offre = $infos[3]; # Potentiellement != de l'offre demandee...
		my $datecr = $infos[39];
		my $datefuture = $infos[40];
		my $newtel = $infos[41];
		my $commentaire = $infos[43];
		# Infos anciennes
		my @ar = @infos[1..38];
		my @ordre = @ar[1..32];
		my $oid = $ordre[2];
		
		my $telephone = $ordre[11];
		my $event_cr = 'CRTD-'.$oid;
		my $event_props = {
			'telephone' => $telephone,
			'statut' => $etat,
			'offre' => $offre,
			'date-cr' => $datecr,
			'date-future' => $datefuture,
			'new-tel' => $newtel,
			'commentaire-td' => $commentaire
			};
		send_event($event_cr,$event_props);
	}
}

sub traite_fichier_notif($) {
}

sub x_error($) {
	error $_[0];
	print STDERR $_[0]."\n";
}


for my $file ( @ARGV ) {
	traite_fichier($file);
}
exit 0;

