#!/usr/bin/perl 

# Debian dependencies : 
# Depends: perl-api, libnet-perl, libgnupg-perl
# Conflicts: placenet

use strict;
use Net::FTP;
use GnuPG;
use FDN::Common::Config;
use FDN::Common::Logger;

# FTP Account properties to connect to Cegetel Server : 
my $ftp_server = Config_Key('adsl/tdftp.cfg','ftp','server');
my $ftp_login  = Config_Key('adsl/tdftp.cfg','ftp','login');
my $ftp_pass   = Config_Key('adsl/tdftp.cfg','ftp','passwd');
# Directory on the remote server : 
my $ftp_dir    = Config_Key('adsl/tdftp.cfg','ftp','indir');

# Local directories : 
# - incoming = where we store crypted files from ftp
# - temp = where we store decrypted files temporarily.
# - spool = where we move decrypted files to be processed by another daemon. 
my $dir_incoming = Config_Key('adsl/global.cfg','dirs','tdin');
my $dir_temp     = Config_Key('adsl/global.cfg','dirs','tdprocess');
my $dir_spool    = Config_Key('adsl/global.cfg','dirs','tdincoming');
my $dir_store    = Config_Key('adsl/global.cfg','dirs','tdstore');

# Gnupg home directory : 
my $gnupg_homedir = Config_Key('adsl/tdftp.cfg','gnupg','homedir');
my $crypt_passphrase = Config_Key('adsl/tdftp.cfg','gnupg','passphrase');


##########################################################
# Private and globals
my @flist; # Remote or Local File List 
my $k;

my $gpg = new GnuPG( homedir => $gnupg_homedir)
    or die "FATAL: Cannot initialise the GnuPG library";

my $ftp = Net::FTP->new($ftp_server)
    or die "FATAL: Cannot connect to the FTP server: $@";

$ftp->login($ftp_login,$ftp_pass)
    or die "FATAL: Cannot login ", $ftp->message;

$ftp->cwd($ftp_dir)
    or die "FATAL: Cannot change working directory ", $ftp->message;

$ftp->binary()
    or die "FATAL: Cannot change to binary mode ", $ftp->message;

@flist = $ftp->ls() 
    or die "FATAL: Cannot list the files on the remote side ", $ftp->message;
 

# Download the remote files : 

print "Found ".(scalar @flist)." files.\n";

foreach $k (@flist) {
	next if ( $k eq '.' );
	next if ( $k eq '..' );
	print "Un fichier $k!\n";
}

$ftp->quit;


