#!/usr/bin/perl 

# Debian dependencies : 
# Depends: perl-api, libnet-perl, libgnupg-perl
# Conflicts: placenet

use strict;
use Net::FTP;
use FDN::Common::Config;
use FDN::Common::Logger;

# FTP Account properties to connect to Cegetel Server : 
my $ftp_server = Config_Key('adsl/tdftp.cfg','ftp9','server');
my $ftp_login  = Config_Key('adsl/tdftp.cfg','ftp9','login');
my $ftp_pass   = Config_Key('adsl/tdftp.cfg','ftp9','passwd');
# Directory on the remote server : 
my $ftp_dir_ar  = Config_Key('adsl/tdftp.cfg','ftp9','indir_ar');
my $ftp_dir_crf = Config_Key('adsl/tdftp.cfg','ftp9','indir_crf');
my $ftp_dir_cr  = Config_Key('adsl/tdftp.cfg','ftp9','indir_cr');
my $ftp_dir_no  = Config_Key('adsl/tdftp.cfg','ftp9','indir_no');

# Local directories : 
# - spool = where we move decrypted files to be processed by another daemon. 
my $dir_spool    = Config_Key('adsl/global.cfg','dirs','9tincoming');

##########################################################
# Private and globals
my @flist; # Remote or Local File List 
my $k;

my $ftp = Net::FTP->new($ftp_server, 'Passive' => 1)
    or die "FATAL: Cannot connect to the FTP server: $@";

$ftp->login($ftp_login,$ftp_pass)
    or die "FATAL: Cannot login ", $ftp->message;

$ftp->binary()
    or die "FATAL: Cannot change to binary mode ", $ftp->message;

sub download_dir($$) {
	my ( $ftp, $dir ) = @_;

	$ftp->cwd('/'.$dir)
	    or die "FATAL: Cannot change working directory ", $ftp->message;

	# Lister les fichiers
	@flist = $ftp->ls(); 
 
	# Les tlcharger
	info "Found ".(scalar @flist)." files.";

	foreach $k (@flist) {
		next if ( $k eq '.' );
		next if ( $k eq '..' );
		info "Downloading $k";
		if (!$ftp->get($k,$dir_spool."/".$k)) {
			error "Failure downloading $k";
			print STDERR "Failure downloading $k\n";
			next;
		}
		info "Downloaded successfuly $k";
		# TODO : add the deletion command
		# (the specifications does not tell us if we must delete the remote files)
		$ftp->rename($k,'/backups/'.$dir.'/'.$k) or
		    print STDERR "Failure renaming $k to ".'/backups/'.$dir.'/'.$k."\n";
		# $ftp->delete($k);
	}
}

download_dir($ftp,$ftp_dir_ar);
#download_dir($ftp,$ftp_dir_cr);
download_dir($ftp,$ftp_dir_crf);
download_dir($ftp,$ftp_dir_no);

$ftp->quit;


