#!/usr/bin/perl 

# Debian dependencies : 
# Depends: perl-api, libnet-perl, libgnupg-perl
# Conflicts: placenet

use strict;
use Net::FTP;
use GnuPG;
use FDN::Common::Config;
use FDN::Common::Logger;


# FTP Account properties to connect to Cegetel Server : 
my $ftp_server = Config_Key('adsl/tdftp.cfg','ftp','server');
my $ftp_login  = Config_Key('adsl/tdftp.cfg','ftp','login');
my $ftp_pass   = Config_Key('adsl/tdftp.cfg','ftp','passwd');
# Directory on the remote server : 
my $ftp_dir    = Config_Key('adsl/tdftp.cfg','ftp','outdir');

# Local directories : 
# - incoming = where we take unencrypted file that will be sent to cegetel.
# - temp = where we store ecnrypted files temporarily. 
# - archive = where we store unencrypted files after the upload.
my $dir_incoming = Config_Key('adsl/global.cfg','dirs','tdout');
my $dir_temp     = Config_Key('adsl/global.cfg','dirs','tdprocess');
my $dir_archive  = Config_Key('adsl/global.cfg','dirs','tdoutdone');
my $dir_store    = Config_Key('adsl/global.cfg','dirs','tdstore');

# Gnupg home directory : 
my $gnupg_homedir = Config_Key('adsl/tdftp.cfg','gnupg','homedir');
my $crypt_recipient = Config_Key('adsl/tdftp.cfg','gnupg','recipient');
my $crypt_passphrase = Config_Key('adsl/tdftp.cfg','gnupg','passphrase');

##########################################################
# Private and globals
my @flist; # Remote or Local File List 
my $k;

my $gpg = new GnuPG( homedir => $gnupg_homedir);
if ( ! $gpg ) {
	error
	die "FATAL: Cannot initialise the GnuPG library";
}

# We walk through the files in $dir_incoming and crypt them.
opendir(DIR,$dir_incoming) or die "FATAL: Can't open the incoming directory\n";
@flist = readdir(DIR) or die "FATAL : unable to read incoming directory\n";

foreach $k (@flist) {
	next if ! ( -f $dir_incoming."/".$k);
	info "Encrypting $k";
	$gpg->encrypt(
		plaintext => $dir_incoming."/".$k,
		output=> $dir_temp."/".$k.'.pgp',
		armor => 'true',
		recipient=>$crypt_recipient,
		sign  => 'true',
		passphrase=>$crypt_passphrase);
	info "Encryption done.";
}

# Now upload them : 
my $ftp = Net::FTP->new($ftp_server)
    or die "FATAL: Cannot connect to the FTP server: $@";

$ftp->login($ftp_login,$ftp_pass)
    or die "FATAL: Cannot login ", $ftp->message;

$ftp->cwd($ftp_dir)
    or die "FATAL: Cannot change working directory ", $ftp->message;

$ftp->binary()
    or die "FATAL: Cannot change to binary mode ", $ftp->message;

# Download the remote files : 

foreach $k (@flist) {
	next if ( ! -f $dir_temp."/".$k.'.pgp');
	my $res = $ftp->put($dir_temp."/".$k.'.pgp',$k.'.pgp');
	if ( ! $res ) {
		error "Failed uploading $k.pgp";
		print STDERR "Failed uploading $k.pgp";
	} else {
		info "Successfuly uploaded $k.pgp";
	}
	rename($dir_incoming."/".$k,    $dir_archive."/".$k);  # move it to archive.
	rename($dir_temp."/".$k.'.pgp', $dir_store."/".$k.'.pgp');
	#unlink($dir_temp."/".$k.'.pgp');
}

$ftp->quit;


