package FDN::Adsl::Adhs;

use strict;
use Exporter;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	prepare_page_adh
	eject_page_adh
	liste_docs
	liste_help
	help_file
	liste_ids
	liste_ips
	liste_stats
	liste_statpg
	$sname
	$sess
	$vars
	$tpldir
);
our @EXPORT_OK = qw (
);
our %EXPORT_TAGS = qw (
);

use CGI;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::Array;
use FDN::Web::Session;
use FDN::Web::TemplateParse qw(parse remplace);
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;

our $cgi;
our $vars;
our $sname;
our $sess;
our $tpldir;
my $hash = {};

sub prepare_page_adh() {
	$cgi = new CGI;
	$vars = $cgi->Vars;
	$sname = $vars->{sess};
	$sess  = retrieve_session('adh', $sname);

	if ( ! defined $sess ) {
		eject_page_error('Session invalide');
		exit(0);
	}

	my $cid = $sess->{content}->{cid};
	my $client = client_par_id($cid);

	$tpldir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	$hash->{ADH} = tableau_1d('adsl/adhs/rewrite.tbl','adherent',$client);
	$hash->{MENU} = remplace($tpldir.'/adhs/menu.tpl',{sess=>$sname},{});
}

sub eject_page_adh($) {	
	my ( $text ) = @_;
	$hash->{INFOS} = parse($text,{sess=>$sname},{});
	my $page = remplace($tpldir.'/adhs/page-adh.tpl',$hash,1);
	eject_page($page);
}

sub liste_docs() {
	my $page = '';
	my $cid = $sess->{content}->{cid};
	my @lignes = ligne_par_client_id($cid);
	for my $lig ( @lignes ) {
		my $lid = $lig->{'LIGNE_ID'};
		my $tel = $lig->{'LIGNE_TEL'};
		$page .= liste_docs_ligne($lid,$tel);
	}
	my $access = adhaccess_par_client_id($cid);
	if ( defined $access ) {
		$page .= liste_docs_access($access);
	}
	return $page;
}

sub liste_docs_ligne($$) {
	my ( $lid, $tel ) = @_;
	my $res = '';
	my $dpl = deploiement_par_ligne_id($lid);
	return if ( ! defined $dpl );
	my $dplnum = $dpl->{'DEPLOIEMENT_NUM'};
	$res .= "<h1>Documents lis  la ligne $tel</h1>";
	$res .= liste_docs_file('ligne',$dplnum,'adhesion.pdf');
	$res .= liste_docs_file('ligne',$dplnum,'autorisation.pdf');
	$res .= liste_docs_file('ligne',$dplnum,'souscription.pdf');
#FIXME
#	$res .= liste_docs_file('mandat',$dplnum,'mandat.pdf');
	return $res;
}

sub liste_docs_access($) {
	my ( $access ) = @_;
	my $res = "<h1>Documents lis  l'adhsion</h1>\n";
	my $login = $access->{'ADHACCESS_LOGIN'};
	$res .= liste_docs_file('access',$login,'adhesion.pdf');
	$res .= liste_docs_file('access',$login,'autorisation.pdf');
	return $res;
}

sub liste_docs_file($$$) {
	my ( $type, $dir, $doc ) = @_;
	my $base = Config_Key('adsl/global.cfg','dirs','stockpdf');
	if ( -f $base.'/'.$dir.'/'.$doc ) {
		return '<div class="doc">'.
		       "<a href=\"print-doc.cgi?sess=$sname&dir=$dir&type=$type&doc=$doc\">$doc</a>".
		       '</div>';
	}
}

sub liste_help() {
	return help_file('menu.tpl');
}

sub help_file($) {
	my ( $file ) = @_;
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	$file = $dir.'/adhs/help-'.$file;
	local $/;
	open FILE, $file;
	my $text = <FILE>;
	close FILE;
	return $text;
}

sub liste_ids() {
	my $radid = liste_ids_radius();
	my $telid = liste_ids_telephone();
	my $adhid = liste_ids_adhaccess();
	my $hash = {
		'radid' => $radid,
		'telid' => $telid,
		'adhid' => $adhid
	};
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	my $tpl = $dir.'/adhs/info-ids.tpl';
	my $res = remplace($tpl,$hash,1);
	return $res;
}

sub liste_ids_telephone() {
	my $cid = $sess->{'content'}->{'cid'};
	my @lignes = ligne_par_client_id($cid);
	my @ids;
	for my $lig ( @lignes ) {
		my $tel = $lig->{'LIGNE_TEL'};
		my $lid = $lig->{'LIGNE_ID'};
		my $depl = deploiement_par_ligne_id($lid);
		my $dplnum = $depl->{'DEPLOIEMENT_NUM'};
		if ( $dplnum ) {
			push @ids, { 'tel' => $tel, 'num' => $dplnum };
		}
	}
	if ( $#ids >= 0 ) {
		return \@ids;
	} else {
		return undef;
	}
}

sub liste_ids_radius() {
	my $cid = $sess->{'content'}->{'cid'};
	my @lignes = ligne_par_client_id($cid);
	my @ids;
	for my $lig ( @lignes ) {
		my $lid = $lig->{'LIGNE_ID'};
		my $user = raduser_par_ligne_id($lid);
		my $uid = $user->{'RADUSER_ID'};
		my $login = $user->{'RADUSER_LOGIN'};
		my $passwd = '';
		my @uattrs = uattr_par_raduser_id_type($uid,'check');
		for my $ua ( @uattrs ) {
			if ( $ua->{'UATTR_ATTR'} eq 'User-Password' ) {
				$passwd = $ua->{'UATTR_VALUE'};
			}
		}
		if ( $login && $passwd ) {
			push @ids, { 'login' => $login, 'passwd' => $passwd };
		}
	}
	if ( $#ids >= 0 ) {
		return \@ids;
	} else {
		return undef;
	}
}

sub liste_ids_adhaccess() {
	my $cid = $sess->{'content'}->{'cid'};
	my $adh = adhaccess_par_client_id($cid);
	if ( defined $adh ) {
		return [{
			'adh' => $adh->{'ADHACCESS_LOGIN'},
			'passwd' => $adh->{'ADHACCESS_PASSWD'}
		}];
	} else {
		return undef;
	}
}

sub liste_ips() {
	my $cid = $sess->{'content'}->{'cid'};
	my @lignes = ligne_par_client_id($cid);
	my @ips;
	for my $lig ( @lignes ) {
		my $lid = $lig->{'LIGNE_ID'};
		my $tel = $lig->{'LIGNE_TEL'};
		my $user = raduser_par_ligne_id($lid);
		my $uid = $user->{'RADUSER_ID'};
		my @ipv4s;
		my @ipv6s;
		my @uattrs = uattr_par_raduser_id_type($uid,'reply');
		for my $ua ( @uattrs ) {
			if ( $ua->{'UATTR_ATTR'} eq 'Framed-IP-Address' ) {
				push @ipv4s, { 'ipv4' => $ua->{'UATTR_VALUE'} };
			# inutile...
			#} elsif ( $ua->{'UATTR_ATTR'} eq 'Framed-IP-Netmask' ) {
			} elsif ( $ua->{'UATTR_ATTR'} eq 'Framed-IPv6-Route' ) {
				push @ipv6s, { 'ipv6' => $ua->{'UATTR_VALUE'} };
			} elsif ( $ua->{'UATTR_ATTR'} eq 'Framed-Route' ) {
				push @ipv4s, { 'ipv4' => $ua->{'UATTR_VALUE'} };
			}
		}
		if ( $#ipv4s >= 0 || $#ipv6s >= 0 ) {
			push @ips, { 'tel' => $tel, 'ipv4s' => \@ipv4s, 'ipv6s' => \@ipv6s };
		}
	}

	my $hash = undef;
	if ( $#ips >= 0 ) {
		$hash = { 'ips' => \@ips };
	}
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	my $tpl = $dir.'/adhs/info-ips.tpl';
	my $res = remplace($tpl,$hash,1);
	return $res;
}

sub liste_stats() {
	my $cid = $sess->{'content'}->{'cid'};
	my $users = $sess->{'content'}->{'users'};
	if ( ! defined $users ) {
		$users = {};
		$users->{'liste'} = [];
		my @lignes = ligne_par_client_id($cid);
		for my $lig ( @lignes ) {
			my $lid = $lig->{'LIGNE_ID'};
			my $user = raduser_par_ligne_id($lid);
			my $login = $user->{'RADUSER_LOGIN'};
			$login =~ s/\@fdn\.fr$/\@fdn\.nerim/;
			$users->{$login} = 1;
			push @{$users->{'liste'}}, $login;
		}
		$sess->{'content'}->{'users'} = $users;
		store_session($sess);
	}
	my $hash = {
		'users' => [],
		'sess' => '%%sess%%'
	};
	conduse('FDN::Web::Forms');
	for my $u ( @{$users->{'liste'}} ) {
		my $ue = FDN::Web::Forms::str_http_encode($u);
		push @{$hash->{'users'}}, { 'user' => $ue, 'clear' => $u, 'sess' => '%%sess%%' };
	}
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	my $tpl = $dir.'/adhs/stats.tpl';
	my $res = remplace($tpl,$hash,1);
	return $res;
	
}

sub liste_statpg() {
	my $users = $sess->{'content'}->{'users'};
	if ( ! defined $users ) {
		error "Rejet session statpg parce-que users n'est pas dans la session.";
		return "Session invalide";
	}
	my $user = $vars->{'user'};
	if ( ! defined $user ) {
		error "Rejet session statpg parce-que user n'est pas dans les variables.";
		return "Session invalide";
	}
	if ( $user ne 'connexions' &&
	     $user ne 'total.rrd' &&
	     $user ne 'total_option1.rrd' &&
	     $user ne 'total_option3.rrd' ) {
		$user =~ s/\.rrd$//;
		if ( ! $users->{$user} ) {
			error "Rejet session statpg parce que $user n'appartient pas  cet adhrent.";
			return "Session invalide";
		}
		$user .= '.rrd';
	}
	conduse('FDN::Web::Forms');
	my $hash = {
		'clear' => $user,
		'user' => FDN::Web::Forms::str_http_encode($user),
		'sess' => $sname
	};
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	my $tpl = $dir.'/adhs/statpg.tpl';
	my $res = remplace($tpl,$hash,1);
	return $res;
}

1;

