package FDN::Adsl::Automate;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		traite_fichier_td
		traite_fichier_9t
		);
our @EXPORT_OK = qw ();

#use XML::XPath;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Sql::Workflow::Basic;
use FDN::Sql::Workflow::Run;
use FDN::Adsl::Basic;

BEGIN {
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');
}

sub x_error($) {
	error $_[0];
	print STDERR "ERR ".$_[0]."\n";
}

sub x_info($) {
	info $_[0];
	print STDERR "INF ".$_[0]."\n";
}
our $debug=0;
sub __debug($) {
	my ( $txt ) = @_;
	if ($debug) {
		debug $txt;
	}
}

sub traite_fichier_td($) {
	my ( $filename ) = @_;
	if ( $filename =~ /\.agp\.arfic$/ ) {
		traite_fichier_arfic($filename);
	} elsif ( $filename =~ /\.agp\.rej$/ ) {
		traite_fichier_rejet($filename);
	} elsif ( $filename =~ /\.agp\.ar$/ ) {
		traite_fichier_artd($filename);
	} elsif ( $filename =~ /\.agp\.cr$/ ) {
		traite_fichier_crtd($filename);
	} elsif ( $filename =~ /\.no$/ ) {
		traite_fichier_notiftd($filename);
	} else {
		x_error "Fichier innatendu: $filename";
	}
}

sub traite_fichier_9t($) {	
	my ( $filename ) = @_;
	if ( $filename !~ /^(?:.*\/)?([A-Za-z0-9]+)_(\d{8})(?:_(nummandat))?_(\d{1,2})_(ARM|ARF|AR|CR|NOT)(?:_(\d{1,6}))?\.xml(?:\.gz)?$/ ) {
		x_error "Fichier innatendu: $filename";
		return;
	}
	my ( $client, $date, $mandat, $seq, $type, $sequniq ) = ( $1, $2, $3, $4, $5, $6 );
	my $cl = Config_Key ( 'adsl/global.cfg', 'fdn', 'codeclient9' );
	if ( $client ne $cl ) {
		x_error "Fichier pas pour moi (client=$client, moi=$cl), je continue quand mme.";
	}
	if ( defined $mandat && $type ne 'ARM' ) {
		x_error "Numro de mandat spcifi pour un fichier type $type. Je continue...";
	}
	if ( defined $sequniq ) {
		x_error "On a un numro de squence unique ($sequniq), pas normal a. Je continue...";
	}
	if ( $type eq 'ARM' ) {
		traite_fichier_arm($filename);
	} elsif ( $type eq 'ARF' ) {
		traite_fichier_arf($filename);
	} elsif ( $type eq 'AR' ) {
		traite_fichier_ar9t($filename);
	} elsif ( $type eq 'CR' ) {
		traite_fichier_cr9t($filename);
	} elsif ( $type eq 'NOT' ) {
		traite_fichier_not($filename);
	} else {
		x_error "Je ne devrai vraiment pas me retrouver l: type=$type.";
	}
}

my $rejet_arm_9t = {
	'PGZ' => "Problme  la dcompression (gzip)",
	'NFI' => "Nom de fichier invalide"
};

sub traite_fichier_arm($) {
	my ( $filename ) = @_;
	require 'XML/XPath.pm';
	my $xp = XML::XPath->new(filename => $filename);
	if ( ! defined $xp ) {
		x_error "Erreur de lecture du fichier $filename par XPath. Arret du traitement.";
		return;
	}
	my ( $tmp, $file ) = @_;
	$tmp = $xp->getNodeText('/echange/@code_echange');
	if ( $tmp ne 'AR_MANDAT' ) {
		x_error "Fichier $filename se prtend ARM sans code_echange=AR_MANDAT. Arret du traitement.";
		return;
	}
	$file = $xp->getNodeText('/echange/ar_mandat/fichier');
	$tmp  = $xp->getNodeText('/echange/ar_mandat/@etat');
	if ( $tmp eq 'OK' ) {
		x_info "Reu un ARM positif ($filename) pour le mandat $file.";
		return;
	} elsif ( $tmp eq 'KO' ) {
		my $err = $xp->getNodeText('/echange/ar_mandat/erreur/@code');
		my $err2;
		if ( defined $rejet_arm_9t->{$err} ) {
			$err2 = $rejet_arm_9t->{$err};
		} else {
			$err2 = 'INCONNU';
		}
		x_error "Reu un ARM ngatif ($filename) pour le mandat $file, code $err ($err2).";
		return;
	} else {
		x_error "Rcu un fichier ARM dont l'tat n'est ni OK ni KO ($tmp). Arret du traitement.";
		return;
	}
}

my $rejet_arf_9t = {
	'NTX' => "Fichier non XML",
	'UKE' => "lment XML inconnu",
	'PGZ' => "Problme  la dcompression (gzip)",
	'NFI' => "Nom de fichier invalide",
	'CCI' => "Code client invalide"
};

sub traite_fichier_arf($) {
	my ( $filename ) = @_;
	require 'XML/XPath.pm';
	my $xp = XML::XPath->new(filename => $filename);
	if ( ! defined $xp ) {
		x_error "Erreur de lecture du fichier $filename par XPath. Arret du traitement.";
		return;
	}
	my ( $tmp, $file );
	$tmp = $xp->getNodeText('/fichier_ar_fichier/echange/@code_echange');
	if ( $tmp ne 'AR_FICHIER' ) {
		x_error "Fichier $filename se prtend ARF sans code_echange=AR_FICHIER. Arret du traitement.";
		return;
	}
	$file = $xp->getNodeText('/fichier_ar_fichier/echange/ar_fichier/fichier');

	my @ordres = ordretd_par_fichier($file);
	if ( array_has_no_rows(\@ordres) ) {
		x_error "Reu un fichier ARF: $filename qui ne correspond pas a des ordres. Arret du traitement.";
		return;
	}
	$tmp = $xp->getNodeText('/fichier_ar_fichier/echange/ar_fichier/@etat');
	if ( $tmp eq 'OK') {
		for my $o ( @ordres ) {
			my $oid = $o->{'ORDRETD_ID'};
			send_event('ARFIC-'.$oid);
		}
		return;
	} elsif ( $tmp eq 'KO' ) {
		my $err = $xp->getNodeText('/fichier_ar_fichier/echange/erreur_code/@code');
		my $cmt = $xp->getNodeText('/fichier_ar_fichier/echange/erreur_code/commentaire');
		x_error "ARF de rejet ($filename) portant sur le fichier $file.";
		my $err2;
		if ( defined $rejet_arf_9t->{$err} ) {
			$err2 = $rejet_arf_9t->{$err};
		} else {
			$err2 = 'INCONNU';
		}
		x_error "Motif de rejet: $err ($err2)";
		x_error "Commentaire: $cmt";
		return;
	} else {
		x_error "Reu un fichier ARF ($filename) dont l'tat n'est ni OK ni KO. Arret du traitement.";
		return;
	}
}

sub traite_fichier_ar9t($) {
	my ( $filename ) = @_;
	require 'XML/XPath.pm';
	my $xp = XML::XPath->new(filename => $filename);
	if ( ! defined $xp ) {
		x_error "Erreur de lecture du fichier $filename par XPath. Arret du traitement.";
		return;
	}
	my ( $tmp, $fdnid );
	$tmp = $xp->getNodeText('/fichier_ar/entete/@code_client');
	$fdnid = Config_Key('adsl/global.cfg', 'fdn', 'numclient9');
	if ( $tmp ne $fdnid ) {
		x_error "Fichier AR 9t ($filename) qui n'a pas $fdnid comme code_client. Arret du traitement.";
		return;
	}
	$tmp = $xp->getNodeText('/fichier_ar/entete/@version');
	if ( $tmp ne '3.5' ) {
		x_error "Fichier AR 9t ($filename) qui n'est pas dans la version connue du protocole. Arret du traitement.";
		return;
	}
	
	my @ars = $xp->findnodes('/fichier_ar/accuse_reception');
	if ( $#ars == -1 ) {
		x_error "Fichier AR 9t ($filename) qui ne contient aucun AR. Curieux mais pas invalide. Arret du traitement.";
		return;
	}
	for my $ar ( @ars ) {
		my $xxp = XML::XPath->new(context => $ar);
		my $type_ar = $xxp->getNodeText('infos_ar/@type_ar');
		my $identifiant_dossier = $xxp->getNodeText('infos_ar/@identifiant_dossier');
		my $date_ar = $xxp->getNodeText('infos_ar/@date_ar');
		my $date_real = $xxp->getNodeText('infos_ar/@date_realisation');
		my $cause_rejet = $xxp->getNodeText('infos_ar/@cause_rejet_ar');
		my $degroupe = $xxp->getNodeText('infos_ar/@degroupe');
		my $codeoffre_technique = $xxp->getNodeText('infos_ar/@codeoffre_technique');
		my $id_mandat = $xxp->getNodeText('infos_generales/@id_mandat');
		my $commentaire = $xxp->getNodeText('commentaire_ar/@commentaire');
		my $oid = $xxp->getNodeText('@id_commande_client');
		my $telephone = $xxp->getNodeText('infos_ligne/@identifiant_ligne');
		my $fournisseur = $xxp->getNodeText('infos_ar/@fournisseur');
		my $conduit = $xxp->getNodeText('infos_ar/@conduit');
		if ( ! $oid ) {
			x_error "Reu un AR 9t qui ne contient pas de rfrence commande FDN (\@id_commande_client). Suivant.";
			next;
		}
		my $ordre = ordretd_par_id($oid);
		if ( ! defined $ordre ) {
			x_error "Reu un AR 9t qui contient une rfrence commande FDN invalide. Suivant.";
			next;
		}
		my $event_ar = 'ARTD-'.$oid;
		my $event_props = {
			'oid' => $oid,
			'telephone' => $telephone,
			'statut' => $type_ar,
			'numcde' => $identifiant_dossier,
			'date-recu' => $date_ar,
			'date-traitee' => $date_ar,
			'date-future' => $date_real,
			'code-rejet' => $cause_rejet,
			'commentaire' => $commentaire,
			'degroupe' => $degroupe,
			'offre' => $codeoffre_technique,
			'mandat' => $id_mandat,
			'fournisseur' => $fournisseur,
			'conduit' => $conduit,
			};
		if ( $type_ar eq 'ACCEPTEE' ) {
			set_ordretd_etat($oid,'ok');
		} elsif ( $type_ar eq 'REFUSEE' ) {
			set_ordretd_etat($oid,'ko');
		} else {
			x_error "Le statut de l'AR 9t est incomprhensible: $type_ar";
			x_error "Dans le fichier: $filename";
			x_error "L'accuse_reception:";
			x_error XML::XPath::XMLParser::as_string($ar);
			next;
		}
		my $ev = event_par_event($event_ar);
		if ( defined($ev) ) {
			x_error "L'vnement pour cet AR 9T existe dj. Je l'annule en supposant que c'est exprs.";
			del_event_par_event($event_ar);
		}
		send_event($event_ar,$event_props);
	}
}

sub traite_fichier_cr9t($) {
	my ( $filename ) = @_;
	
	require 'XML/XPath.pm';
	my $xp = XML::XPath->new(filename => $filename);
	if ( ! defined $xp ) {
		x_error "Erreur de lecture du fichier $filename par XPath. Arret du traitement.";
		return;
	}
	my ( $tmp, $fdnid );
	$tmp = $xp->getNodeText('/fichier_cr/entete/@code_client');
	$fdnid = Config_Key('adsl/global.cfg', 'fdn', 'numclient9');
	if ( $tmp ne $fdnid ) {
		x_error "Fichier CR 9t ($filename) qui n'a pas $fdnid comme code_client. Arret du traitement.";
		return;
	}
	$tmp = $xp->getNodeText('/fichier_cr/entete/@version');
	if ( $tmp ne '3.5' ) {
		x_error "Fichier CR 9t ($filename) qui n'est pas dans la version connue du protocole. Arret du traitement.";
		return;
	}
	
	my @crs = $xp->findnodes('/fichier_cr/compte_rendu');
	if ( $#crs == -1 ) {
		x_error "Fichier CR 9t ($filename) qui ne contient aucun CR. Curieux mais pas invalide. Arret du traitement.";
		return;
	}
	for my $cr ( @crs ) {
		my $xxp = XML::XPath->new(context => $cr);
		my $type_cr   = $xxp->getNodeText('infos_cr/@type_cr');
		my $date_cr   = $xxp->getNodeText('infos_cr/@date_cr');
		my $etat      = $xxp->getNodeText('infos_cr/@etat_service');
		my $date_real = $xxp->getNodeText('infos_cr/@date_realisation');
		my $cause_rej = $xxp->getNodeText('infos_cr/@cause_rejet_cr');
		my $comment   = $xxp->getNodeText('commentaire_cr/@commentaire');
		my $id_acces  = $xxp->getNodeText('infos_cr/@id_acces');
		my $degroupe  = $xxp->getNodeText('infos_cr/@degroupe');
		my $offre     = $xxp->getNodeText('infos_cr/@codeoffre_technique');
		my $type_ft   = $xxp->getNodeText('infos_cr/@type_cmd_ft');
		my $telephone = $xxp->getNodeText('infos_ligne/@identifiant_ligne');
		my $oid       = $xxp->getNodeText('@id_commande_client');
		my $identifiant_dossier = $xxp->getNodeText('infos_ar2/@identifiant_dossier');
		my $ref_factu = $xxp->getNodeText('infos_cr/@ref_facturation');
		my $ref_elig  = $xxp->getNodeText('infos_generales/@reference_eligibilite');
		my $id_mandat = $xxp->getNodeText('infos_generales/@id_mandat');

		if ( ! $oid ) {
			x_error "Reu un CR 9T qui ne contient pas de rfrence commande FDN (\@id_commande_client). Suivant.";
			next;
		}
		my $ordre = ordretd_par_id($oid);
		if ( ! defined $ordre ) {
			x_error "Reu un CR 9T qui contient une rfrence commande FDN invalide. Suivant.";
			next;
		}
		my $event_cr = 'CRTD-'.$oid;
		my $event_props = {
			'oid' => $oid,
			'telephone' => $telephone,
			'statut' => $type_cr,
			'numcde' => $identifiant_dossier,
			'date-recu' => $date_cr,
			'date-traitee' => $date_real,
			'etat-service' => $etat,
			'code-rejet' => $cause_rej,
			'commentaire' => $comment,
			'id-acces' => $id_acces,
			'degroupe' => $degroupe,
			'type-cde-ft' => $type_ft,
			'offre-td' => $offre,
			'ref-factu' => $ref_factu,
			'ref-elig' => $ref_elig,
			'id-mandat-td' => $id_mandat
		};
		my $ev = event_par_event($event_cr);
		if ( defined($ev) ) {
			x_error "L'vnement pour ce CR 9T existe dj. Je l'annule en supposant que c'est exprs.";
			del_event_par_event($event_cr);
		}
		send_event($event_cr,$event_props);
	}
}


sub traite_fichier_arfic($) {
	my ( $filename ) = @_;

	if ( ! open IN,"<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $line = <IN>;
	close IN;

	$line =~ /^(\d{8});([^;]*);(\d*)/;
	my ( $date, $file, $lignes ) = ( $1, $2, $3 );
	$file =~ s/\.arfic$//;
	
	my @ordres = ordretd_par_fichier($file);
	if ( array_has_no_rows(\@ordres) ) {
		x_error "Recu un fichier ARFIC: $filename qui ne correspond par a des ordres";
		return;
	}
	if ( $#ordres+2 != $lignes ) {
		x_error "J'ai envoye ".($#ordres+2)." lignes dont une d'en-tete. TD dit n'en avoir recu que $lignes";
	}
	# Lire le fichier, tout de meme!
	for my $o ( @ordres ) {
		my $oid = $o->{'ORDRETD_ID'};
		send_event('ARFIC-'.$oid);
	}
}

my $rejet_fichier = {
	'E0'  => "Mode d'echange incompatible avec le contrat",
	'E1'  => "Entete du fichier pas au bon format",
	'E2'  => "Valeur du champ operateur inconnu",
	'E3'  => "Erreur dans l'entete du fichier: format de date invalide",
	'E4'  => "Erreur dans l'entete du fichier: domaine inconnu",
	'E5'  => "Nom de fichier invalide",
	'E6'  => "Erreur dans le nom du fichier: date au format invalide",
	'E7'  => "Erreur dans le nom du fichier: code operateur invalide",
	'E8'  => "Erreur dans le nom du fichier: numero de sequence au format invalide",
	'E9'  => "Erreur dans le nom du fichier: suffixe invalide",
	'E10' => "Nombre de lignes maximal depasse",
	'E11' => "Code operateur different entre nom du fichier et en-tete",
	'E12' => "Domaine incompatible avec le type de fichier",
	'E13' => "Nombre de champs invalide dans une ligne du fichier",
	'E14' => "Code operateur inconnu (repertoire de reception non identifie)",
	'E15' => "Fichier deja recu",
	'E16' => "Impossible de decrypter le fichier",
	'E17' => "Un champ comporte plus de 512 caracteres",
	'E18' => "Protocole inconnu"
};

sub traite_fichier_rejet($) {
	my ( $filename ) = @_;
	x_error "Un fichier a ete rejete par TD";

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne = <IN>;
	close IN;

	$ligne =~ /^(\d{8});([^;]*);([A-Z]\d+)/;
	my ( $date, $file, $err ) = ( $1, $2, $3 );
	
	if ( ! defined $file ) {
		x_error "Pas russi  comprendre le nom du fichier... J'arrete les frais.";
		return;
	}
	x_error "Fichier en erreur: $file";
	
	if ( ! defined $rejet_fichier->{$err} ) {
		x_error "Code d'erreur <$err> inconnu au bataillon. J'arrete l.";
		return;
	}
	x_error "Code d'erreur: ".$err." (".$rejet_fichier->{$err}.")";
	
	if ( $err eq 'E15' ) {
		x_error "TD dit avoir deja recu le fichier, je ne change rien!";
	} else {
		x_error "L'erreur est plus grave, je marque les ordres en attente.";
		my @ordres = ordretd_par_fichier($file);
		if ( array_has_no_rows(\@ordres) ) {
			x_error "Aucun ordre ne correspond a ce fichier en erreur!";
			return;
		}
		for my $o ( @ordres ) {
			my $oid = $o->{'ORDRETD_ID'};
			if ( ! $oid ) {
				x_error "Curieux: un ordre attach au fichier n'a pas d'identifiant valide... Suivant.";
				next;
			}
			x_error "Je retabli l'ordre numero $oid qui disait:";
			x_error $o->{'ORDRETD_LIGNE'};
			set_ordretd_etat($oid,'new');
			set_ordretd_fichier($oid,'');
		}
	}
}

sub traite_fichier_artd($) {
	my ( $filename ) = @_;

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne;
	my $head = <IN>;
	while ( $ligne = <IN> ) {
		chomp $ligne;
		$ligne =~ s/\r$//;
		my @infos = split /;/, $ligne;
		# Les infos nouvelles
		my $recevabilite = $infos[0];
		my $numero_commande = $infos[33];
		my $date_recu = $infos[34];
		my $date_traitee = $infos[35];
		my $code_rejet = $infos[36];
		my $commentaire = $infos[37];
		# L'ordre de depart
		my @ordre = @infos[1..32];
		my $ligne_orig = join ';', @ordre;
		my $oid = $ordre[2];
		my $telephone = $ordre[11];
		my $ordre = ordretd_par_id($oid);
		if ( $ligne_orig ne $ordre->{'ORDRETD_LIGNE'} ) {
			x_error "La reponse ne colle pas absolument avec la question.";
			x_error "La question etait:";
			x_error $ordre->{'ORDRETD_LIGNE'};
			x_error "La reponse porte sur:";
			x_error $ligne_orig;
		}
		my $event_ar = 'ARTD-'.$oid;
		my $event_props = {
			'oid' => $oid,
			'telephone' => $telephone,
			'statut' => $recevabilite,
			'numcde' => $numero_commande,
			'date-recu' => $date_recu,
			'date-traitee' => $date_traitee,
			'code-rejet' => $code_rejet,
			'commentaire' => $commentaire
			};
		if ( $recevabilite eq 'TRANSMISE' ) {
			set_ordretd_etat($oid,'ok');
		} elsif ( $recevabilite eq 'REJETEE' ) {
			set_ordretd_etat($oid,'ko');
		} else {
			x_error "Le status de l'AR est incomprehensible: $recevabilite";
			x_error "Dans le fichier: $filename";
			x_error "La ligne:";
			x_error $ligne;
			return;
		}
		my $ev = event_par_event($event_ar);
		if ( defined($ev) ) {
			x_error "L'vnement pour cet AR TD existe dj. Je l'annule en supposant que c'est exprs.";
			del_event_par_event($event_ar);
		}
		send_event($event_ar,$event_props);
	}
}

sub traite_fichier_crtd($) {
	my ( $filename ) = @_;

	if ( ! open IN, "<$filename" ) {
		x_error "Echec d'ouverture du fichier $filename: $!";
		return;
	}
	my $ligne;
	my $head = <IN>;
	while ( $ligne = <IN> ) {
		chomp $ligne;
		$ligne =~ s/\r$//;
		my @infos = split /;/, $ligne;
		# Les infos nouvelles
		my $etat = $infos[0];
		my $offre = $infos[3]; # Potentiellement != de l'offre demandee...
		my $datecr = $infos[39];
		my $datefuture = $infos[40];
		my $newtel = $infos[41];
		my $commentaire = $infos[43];
		# Infos anciennes
		my @ar = @infos[1..38];
		my @ordre = @ar[1..32];
		my $oid = $ordre[2];
		
		my $telephone = $ordre[11];
		my $event_cr = 'CRTD-'.$oid;
		my $event_props = {
			'telephone' => $telephone,
			'statut' => $etat,
			'offre' => $offre,
			'date-cr' => $datecr,
			'date-future' => $datefuture,
			'new-tel' => $newtel,
			'commentaire-td' => $commentaire
			};
		my $ev = event_par_event($event_cr);
		if ( defined $ev ) {
			x_error "L'vnnement pour ce CR TD existe dj. Je l'annule en supposant que c'est exprs.";
			del_event_par_event($event_cr);
		}
		send_event($event_cr,$event_props);
	}
}

# Les notifications anormales, qu'on envoit au bureau

sub __traite_not_nod($$) {
	my ( $filename, $xp ) = @_;
	# Dgroupage partiel -> total
	#FIXME: Il faut envoyer un mail au bureau, c'est anormal.
	my $type = 'bureau-not-nod';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $tel2       = $xp->getNodeText('notification/ndi/@final');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $param      = $xp->getNodeText('notification/parametre_action');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'tel2' => $tel2,
		'date-effet' => $date_effet,
		'param' => $param,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename
	};
	x_info "Notification type NOD (dgroupage partiel -> total) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de degroupage partiel->total";
	}
}

sub __traite_not_nof($$) {
	my ( $filename, $xp ) = @_;
	# Dgroupage total -> partiel
	#FIXME: Il faut envoyer un mail au bureau, c'est anormal.
	my $type = 'bureau-not-nof';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $tel2       = $xp->getNodeText('notification/ndi/@final');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $param      = $xp->getNodeText('notification/parametre_action');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'tel2' => $tel2,
		'date-effet' => $date_effet,
		'param' => $param,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename
	};
	x_info "Notification type NOF (degroupage total -> partiel) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de degroupage total->partiel";
	}
}

sub __traite_not_chc($$) {
	my ( $filename, $xp ) = @_;
	# Changement de caractristique
	#FIXME: Sais pas quoi faire... Mail au bureau.
	my $type = 'bureau-not-chc';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $param      = $xp->getNodeText('notification/parametre_action');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $atm_vc     = $xp->getNodeText('notification/vc');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'param' => $param,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename,
		'atm-vc' => $atm_vc
	};
	x_info "Notification type CHC (changement de caractristiques) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de changement de caractristiques.";
	}
}

sub __traite_not_trk($$) {
	my ( $filename, $xp ) = @_;
	# Notification livraison du modem
	#FIXME: Mail au bureau, c'est forcment une erreur Neuf.
	my $type = 'bureau-not-trk';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $code_expe  = $xp->getNodeText('notification/modem/@code_expedition');
	my $date_expe  = $xp->getNodeText('notification/modem/@date_expedition');
	my $code_trans = $xp->getNodeText('notification/modem/@code_transporteur');
	my $num_track  = $xp->getNodeText('notification/modem/@num_tracking');
	my $ref_prod   = $xp->getNodeText('notification/modem/@ref_produit');
	my $num_ser    = $xp->getNodeText('notification/modem/@num_serie_modem');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename,
		'code-expe' => $code_expe,
		'date-expe' => $date_expe,
		'code-trans' => $code_trans,
		'num-track' => $num_track,
		'ref-prod' => $ref_prod,
		'num-ser' => $num_ser
	};
	x_info "Notification type TRK (livraison du modem) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif d'envoi de modem.";
	}
}

sub __traite_not_ret($$) {
	my ( $filename, $xp ) = @_;
	# Notification retour modem
	#FIXME: Mail au bureau, c'est forcment une erreur Neuf.
	my $type = 'bureau-not-ret';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $code_expe  = $xp->getNodeText('notification/modem/@code_expedition');
	my $num_ser    = $xp->getNodeText('notification/modem/@num_serie_modem');
	my $date_ret   = $xp->getNodeText('notification/modem/@date_retour');
	my $motif_ret  = $xp->getNodeText('notification/modem/@motif_retour');
	my $num_rma    = $xp->getNodeText('notification/modem/@num_rma');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename,
		'code-expe' => $code_expe,
		'num-ser' => $num_ser,
		'date-ret' => $date_ret,
		'motif-ret' => $motif_ret,
		'num-rma' => $num_rma
	};
	x_info "Notification type RET (retour du modem) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de retour de modem.";
	}
}

sub __traite_not_ndb($$) {
	my ( $filename, $xp ) = @_;
	# Notification de bundle
	#FIXME: Mail au bureau, personne sait ce que c'est.
	my $type = 'bureau-not-ndb';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename
	};
	x_info "Notification type NDB (notification de bundle) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de bundle.";
	}
}

sub __traite_not_neo($$) {
	my ( $filename, $xp ) = @_;
	# Notification tude OK
	#FIXME: Mail au bureau, on sait pas dans quel cas a peut arriver
	my $type = 'bureau-not-neo';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $id_cde     = $xp->getNodeText('notification/@id_commande_client');
	my $ref_uti    = $xp->getNodeText('notification/@reference_utilisateur');
	my $atm_vc     = $xp->getNodeText('notification/vc');
	my $date_real  = $xp->getNodeText('notification/@date_realisation_prevue');
	my $degroupe   = $xp->getNodeText('notification/@degroupe');
	my $offre      = $xp->getNodeText('notification/@codeoffre_technique');
	my $date_rdv   = $xp->getNodeText('notification/@date_rdv_ft');
	my $time_rdv   = $xp->getNodeText('notification/@heure_rdv_ft');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'commentaire' => $comment,
		'id-cde' => $id_cde,
		'ref-uti' => $ref_uti,
		'fichier' => $filename,
		'atm-vc' => $atm_vc,
		'date-real' => $date_real,
		'degroupe' => $degroupe,
		'offre' => $offre,
		'date_rdv' => $date_rdv,
		'time_rdv' => $time_rdv
	};
	x_info "Notification type NEO (tude OK) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif d'tude OK.";
	}
}

sub __traite_not_no($$) {
	my ( $filename, $xp ) = @_;
	# Notification autre (amusant, non?)
	#FIXME: Mail au bureau pour traitement manuel.
	my $type = 'bureau-not-no';
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $task_vars = {
		'type' => $type,
		'telephone' => $telephone,
		'fichier' => $filename
	};
	x_info "Notification NO (autre) sur ligne $telephone.";
	my $tid = launch_task('mail-bur',$task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche d'envoie de mail au bureau";
		x_error "Dans le fichier $filename, j'ai reu un notif de type inconnu (ou 'NO' ce qui ne vaut pas plus lourd).";
	}
}

# Les notifications hors-dploiement

sub __traite_not_non($$) {
	my ( $filename, $xp ) = @_;
	# Changement de numro
	#FIXME: Il faut lancer un process (mise  jour de la ligne de l'adhrent), prvenir tout
	#le monde. Mail au bureau.
	my $oldnum = $xp->getNodeText('notification/ndi/@initial');
	my $newnum = $xp->getNodeText('notification/ndi/@final');
	my $task_vars = {
		'oldnum' => $oldnum,
		'newnum' => $newnum,
		'fichier' => $filename
	};
	x_info "Notification NON (changement de numro) sur ligne $oldnum ($newnum).";
	my $tid = launch_task('notif-num', $task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche de changement de numro.";
		x_error "Dans le fichier $filename, on m'a dit que le numro <$oldnum> devient <$newnum>.";
	}
}

sub __traite_not_nos($$) {
	my ( $filename, $xp ) = @_;
	# Notification de perte d'accs.
	#FIXME: Il faut lancer un process, Lequel? Comment? Simplement un mail au bureau?
	my $date_effet = $xp->getNodeText('notification/@date_effet');
	my $telephone  = $xp->getNodeText('notification/ndi/@initial');
	my $param      = $xp->getNodeText('notification/parametre_action');
	my $comment    = $xp->getNodeText('notification/commentaire');
	my $idcde      = $xp->getNodeText('notification/@id_commande_client');
	my $refuti     = $xp->getNodeText('notification/@reference_utilisateur');
	my $task_vars = {
		'telephone' => $telephone,
		'date-effet' => $date_effet,
		'param'      => $param,
		'comment'    => $comment,
		'idcde'      => $idcde,
		'refuti'     => $refuti,
		'fichier'    => $filename
	};
	x_info "Notification type NOS (perte d'accs sur ligne $telephone.";
	my $tid = launch_task('notif-perte', $task_vars);
	if ( ! $tid ) {
		x_error "Echec au lancement de la tche de perte d'accs.";
		x_error "Dans le fichier $filename, on m'a dit que le numro <$telephone> avait rsili.";
	}
}

# Les notifications pendant le dploiement

sub __traite_not_noe($$) {
	my ( $filename, $xp ) = @_;
	# Notification d'envoie  FT
	#FIXME: Il faut prvenir la tche de dploiement pour qu'elle envoie les mails qui vont bien.
}

sub __traite_not_nob($$) {
	my ( $filename, $xp ) = @_;
	# Notification manque de ressource
	#FIXME: Prvenir la tche de dploiement pour qu'elle envoie les mails qui vont bien.
}

sub __traite_not_noa($$) {
	my ( $filename, $xp ) = @_;
	# Notification lancement workflow technique
	#FIXME: Prvenir la tche de dploiement pour qu'elle envoie des mail. Assimilable  une erreur?
}

sub __traite_not_nom($$) {
	my ( $filename, $xp ) = @_;
	# Notification d'absence de mandat
	#FIXME: Envoyer un mail au bureau pour traitement manuel
}

sub __traite_not_nor($$) {
	my ( $filename, $xp ) = @_;
	# Notification de rservation de ressources
	#FIXME: Prvenir la tche de dploiement pour qu'elle envoie les mails qui vont bien.
}

sub __traite_not_nsa($$) {
	my ( $filename, $xp ) = @_;
	# Notification de service actif
	#FIXME: Prvenir la tche de dploiement pour qu'elle envoi les mails qui vont bien.
}

sub traite_fichier_not($) {
	my ( $filename ) = @_;
	require 'XML/XPath.pm';
	my $xp = XML::XPath->new(filename => $filename);
	if ( ! defined $xp ) {
		x_error "Erreur de lecture du fichier $filename par XPath. Arret du traitement.";
		return;
	}
	__debug "$filename est en XML lisible.";
	my ( $tmp, $fdnid );
	$tmp = $xp->getNodeText('/fichier_not/entete/@code_client');
	$fdnid = Config_Key('adsl/global.cfg', 'fdn', 'numclient9');
	if ( $tmp ne $fdnid ) {
		x_error "Fichier NOT 9T ($filename) qui n'a pas $fdnid comme code_client. Arret du traitement.";
		return;
	}
	__debug "$filename est bien pour FDN.";
	$tmp = $xp->getNodeText('/fichier_not/entete/@version');
	if ( $tmp ne '3.5' ) {
		x_error "Fichier NOT 9T ($filename) qui n'est pas dans la version connue du protocole. Arret du traitement.";
		return;
	}
	__debug "$filename est dans la bonne version du protocole.";
	my @nots = $xp->findnodes('/fichier_not/echange');
	if ( $#nots == -1 ) {
		x_error "Fichier NOT 9T ($filename) qui ne contient aucune notification. Curieux et invalide. Arret du traitement.";
		return;
	}
	__debug "$filename contient ".($#nots+1)." notifications.";
	for my $not ( @nots ) {
		my $xxp = XML::XPath->new(context => $not);
		my $idld = $xxp->getNodeText('@id_ldcom');
		my $code = $xxp->getNodeText('@code_echange');
		my $act  = $xxp->getNodeText('notification/@code_action');
		__debug "Notification de type $act.";
		if ( $act eq 'NOS' ) {
			# Perte d'accs
			__traite_not_nos($filename,$xxp);
		} elsif ( $act eq 'NOD' ) {
			# Dgroupage partiel -> total
			__traite_not_nod($filename,$xxp);
		} elsif ( $act eq 'NOF' ) {
			# Dgroupage total -> partiel
			__traite_not_nof($filename,$xxp);
		} elsif ( $act eq 'NON' ) {
			# Changement de numro
			__traite_not_non($filename,$xxp);
		} elsif ( $act eq 'CHC' ) {
			# Changement de caractrisque
			__traite_not_chc($filename,$xxp);
		} elsif ( $act eq 'NOE' ) {
			# Notification d'envoi  FT
			__traite_not_noe($filename,$xxp);
		} elsif ( $act eq 'NOB' ) {
			# Manque de ressource
			__traite_not_nob($filename,$xxp);
		} elsif ( $act eq 'NOA' ) {
			# Lancement du workflow technique (a sent l'interv manuelle  plein nez...)
			__traite_not_noa($filename,$xxp);
		} elsif ( $act eq 'NOM' ) {
			# Absence de mandat
			__traite_not_nom($filename,$xxp);
		} elsif ( $act eq 'NOR' ) {
			# Notif rservation de ressources
			__traite_not_nor($filename,$xxp);
		} elsif ( $act eq 'NSA' ) {
			# Notif service actif
			__traite_not_nsa($filename,$xxp);
		} elsif ( $act eq 'TRK' ) {
			# Notif livraison du modem (on va viter...)
			__traite_not_trk($filename,$xxp);
		} elsif ( $act eq 'RET' ) {
			# Notif retour modem (on va viter aussi...)
			__traite_not_ret($filename,$xxp);
		} elsif ( $act eq 'NDB' ) {
			# Notif de bundle (kzako?)
			__traite_not_ndb($filename,$xxp);
		} elsif ( $act eq 'NEO' ) {
			# Notif d'tude OK
			__traite_not_neo($filename,$xxp);
		} elsif ( $act eq 'NO' ) {
			# Autre notification (on est dans la merde!)
			__traite_not_no($filename,$xxp);
		} else {
			# Notif inconnue....
			x_error "Dans le fichier $filename on a reu une notification d'un type inconnu ($act)";
			x_error "Traiter  la main...";
		}
	}
}


1;
