package FDN::Adsl::Factu;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		cloture_mois
		creation_factures
		creation_facture
		creation_prelevement
		envoyer_prelevement
		encaisser_facture
		encaisser_prelevement
		encaisser_prllig
		encaisser_paiement
		changer_etat_paiement
		rejeter_prllig
		@jours
		);
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Constantes;
use FDN::Common::Modules;
use FDN::Adsl::Basic;
use POSIX qw(strftime);

our @jours = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}


#sub facture_creance($$) {
#	my ( $fid, $creance ) = @_;
#	my $crid = $creance->{'CREANCE_ID'};
#	return FALSE if ( !defined $crid );
#	set_creance_facture($crid,$fid);
#	set_creance_etat($crid,'facturee');
#	return TRUE;
#}

sub cloture_mois($$) { # Test
	my ( $annee, $mois ) = @_;
	my $cloture = cloture_par_date($annee.$mois);
	if ( defined $cloture ) {
		if (!annuler_cloture($cloture)) {
			error "Pas moyen d'annuler la cloture precedente pour le moi $annee / $mois";
			return 0;
		}
	}
	my $clid = new_cloture($annee.$mois);
	if ( ! $clid ) {
		error "Je n'arrive pas a creer une cloture";
		return 0;
	}
	my @clients = tous_clients_actifs();
	return if ( array_has_no_rows(\@clients) );
	for my $cl ( @clients ) {
		cloture_mois_client($clid, $annee, $mois, $cl);
	}
	return $clid;
}

sub cloture_mois_client($$$$) { # Test
	my ( $clid, $annee, $mois, $client ) = @_;
	my $cid = $client->{'CLIENT_ID'};
	info 'Cloture du mois '.$annee.'/'.$mois.' pour le client '.$cid;
	my @ls = ligne_par_client_id($cid);
	for my $ligne ( @ls ) {
		cloture_mois_ligne($clid, $annee, $mois, $client, $ligne);
	}
	my @ps = payant_par_client_id($cid);
	for my $payant ( @ps ) {
		cloture_mois_payant($clid, $annee, $mois, $client, $payant);
	}
}

sub cloture_mois_ligne($$$$$) { # Test
	my ( $clid, $annee, $mois, $client, $ligne ) = @_;
	my ( $ht, $ttc, $libelle );

	info 'Cloture du mois '.$annee.'/'.$mois.' pour la ligne '.$ligne->{'LIGNE_TEL'};
	my $prix = prix_par_id($ligne->{PRIX_ID});
	my $cid  = $client->{CLIENT_ID};
	my $tva  = $prix->{PRIX_TVA};
                
	if (if_fas($ligne,$annee,$mois)) {
		$ttc = $prix->{PRIX_FASTTC};
		$ht  = $prix->{PRIX_FASHT};
		$libelle = "FAS ".$ligne->{LIGNE_TEL};
		my $crid = new_creance($cid, $clid, $ht, $ttc, $tva, $libelle);
		dater_creance($crid,$annee,$mois);
	}
	my $prorata=ligne_prorata($ligne,$annee,$mois);
	if ($prorata>0) {
		$ttc = $prorata * $prix->{PRIX_ABOTTC};
		$ttc =~ s/\..*$//;
		$ht  = $prorata * $prix->{PRIX_ABOHT};
		$ht  =~ s/\..*$//;
		$libelle = "Abo ".$ligne->{LIGNE_TEL}." $annee/$mois";
		my $crid = new_creance($cid, $clid, $ht, $ttc, $tva, $libelle);
		dater_creance($crid,$annee,$mois);
	}
}

sub cloture_mois_payant($$$$$) { # Test
	my ( $clid, $annee, $mois, $client, $payant ) = @_;

	info 'Cloture du mois '.$annee.'/'.$mois.' pour le payant '.$payant->{'PAYANT_LIBELLE'};
	my $prix    = prix_par_id($payant->{'PRIX_ID'});
	my $cid     = $client->{'CLIENT_ID'};
	my $pid     = $payant->{'PAYANT_ID'};
	my $type    = $payant->{'PAYANT_TYPE'};
	my $date    = $payant->{'PAYANT_DATE'};
	my $debut   = $payant->{'PAYANT_DEBUT'};
	my $fin     = $payant->{'PAYANT_FIN'};
	my $libelle = $payant->{'PAYANT_LIBELLE'};
	my $tva     = $prix->{'PRIX_TVA'};
	my $fasht   = $prix->{'PRIX_FASHT'};
	my $fasttc  = $prix->{'PRIX_FASTTC'};
	my $aboht   = $prix->{'PRIX_ABOHT'};
	my $abottc  = $prix->{'PRIX_ABOTTC'};
	if ( $type !~ /^(unique|mensuel|annuel)$/ ) {
		error "J'essaye de facturer un type de PAYANT ($type) que je ne connais pas";
		return;
	}

	my $suffix;
	
	$suffix = {
		'unique' => '',
		'mensuel' => ' (FAS)',
		'annuel'  => ' (1ere annee)' };
	if ( $type eq 'unique' ) {
		$debut = $date;
	}

	if ( if_ce_mois_ci($debut,$annee,$mois) ) {
		my $crid = new_creance($cid, $clid, $fasht, $fasttc, $tva,
			$libelle.$suffix->{$type});
		dater_creance($crid,$annee,$mois);
	}

	if ( $type eq 'unique' ) {
		return;
	}
	
	$suffix = {
		'unique' => '',
		'mensuel' => '',
		'annuel'  => ' (annuel)' };
	if ( payant_abo($type,$debut,$fin,$annee,$mois) ) {
		my $crid = new_creance($cid, $clid, $aboht, $abottc, $tva,
			$libelle.$suffix->{$type});
		dater_creance($crid,$annee,$mois);
	}
}

sub dater_creance($$$) { # Test
	my ( $crid, $annee, $mois ) = @_;
	if ( $crid ) {
		set_creance_datedue($crid, $annee.$mois.$jours[$mois-1].'000000');
	} else {
		error "Je n'arrive pas a creer une creance!";
	}
}

sub annuler_cloture($) { # Test
	my ( $cloture ) = @_;
	my $clid = $cloture->{'CLOTURE_ID'};
	my @crs = creance_par_cloture_id($clid);
	for my $cr ( @crs ) {
		if ( ! annuler_creance($cr) ) {
			error "Pas moyen d'annuler la creance ".$cr->{'CREANCE_ID'};
			return 0;
		}
	}
	del_cloture($clid);
	return 1;
}

sub annuler_creance($) { # Test
	my ( $creance ) = @_;
	my $crid = $creance->{'CREANCE_ID'};
	if ( $creance->{'CREANCE_ETAT'} !~ /^(due|facturee)$/ ) {
		error "La creance $crid est dans un etat tel qu'elle ne peut pas etre annulee";
		return 0;
	}
	if ( $creance->{'CREANCE_ETAT'} eq 'facturee' ) {
		set_creance_facture($crid,-1);
	}
	set_creance_etat($crid,'annulee');
	set_creance_cloture($crid,-1);
	return 1;
}

sub creation_factures() { # Test
	my @cls = tous_clients_facture();
	my $nbfact = 0;
	for my $cl ( @cls ) {
		my $clid = $cl->{'CLIENT_ID'};
		my @crs = creance_par_client_id_etat($clid,'due');
		next if ( $#crs == -1 );
		my $fid = creation_facture($clid, \@crs);
		if ( $fid ) {
			$nbfact++;
		} else {
			error "Erreur a la creation d'une facture...";
		}
	}
	return $nbfact;
}

sub creation_facture($$) { # Test
	my ( $cid, $crs ) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
	$mon++;
	$mon =~ s/^(.)$/0$1/;
	
	my $fnum = strftime "%Y%m%d-%H%M%S", localtime;
	$fnum .= '-'.$cid;
	my $fid = new_facture($cid,$fnum);
	for my $cr ( @$crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		set_creance_facture($crid,$fid);
		set_creance_etat($crid,'facturee');
	}
	return $fid;
}

sub creation_prelevement($$$) { # Test
	my ( $nom, $fichier, $date ) = @_;
	my $pid;

	my @crs;
	my @cls = tous_clients_prelevement();
	for my $cl ( @cls ) {
		my $clid = $cl->{'CLIENT_ID'};
		my $prid = $cl->{'PRELEVEMT_ID'};
		my $prl  = prelevemt_par_id($prid);
		next if ( $prl->{'PRELEVEMT_ETAT'} ne 'ok' );
		@crs = creance_par_client_id_etat($clid,'due');
		next if ( array_has_no_rows(\@crs) );
		my $total = 0;
		for my $cr ( @crs ) {
			my $crid = $cr->{'CREANCE_ID'};
			set_creance_etat($crid,'facturee');
			$total += $cr->{'CREANCE_TTC'};
		}
		if ( $total == 0 ) {
			for my $cr ( @crs ) {
				my $crid = $cr->{'CREANCE_ID'};
				set_creance_etat($crid,'payee');
			}
		} elsif ( $total > 0 ) {
			if ( ! defined $pid ) {
				$pid = new_prlbq($nom,$fichier,$date);
			}
			my $ligid = new_prllig($pid,
				$clid,
				$prl->{'PRELEVEMT_BANQUE'},
				$prl->{'PRELEVEMT_GUICHET'},
				$prl->{'PRELEVEMT_COMPTE'},
				$prl->{'PRELEVEMT_DOMICIL'},
				$prl->{'PRELEVEMT_TITULAIRE'},
				$total,
				$clid.' - '.$nom
			);
			for my $cr ( @crs ) {
				my $crid = $cr->{'CREANCE_ID'};
				set_creance_prllig($crid,$ligid);
			}
		} else {
			info "On doit encore de l'argent a ce type-la: $clid";
		}
	}
	return $pid;
}

sub envoyer_prelevement($$) { # Test
	my ( $pid, $bqdst ) = @_;
	my $pr = prlbq_par_id($pid);
	my @ls = prllig_par_prlbq_id($pid);

	conduse('FDN::Common::Config');
	my $cfg = Config_Section('adsl/global.cfg',"banque-$bqdst");
	if ( ! defined $cfg ||
	     ! defined $cfg->{'emetteur'} ||
	     ! defined $cfg->{'raison-soc'} ||
	     ! defined $cfg->{'banque'} ||
	     ! defined $cfg->{'guichet'} ||
	     ! defined $cfg->{'compte'} ||
	     ! defined $cfg->{'domicil'} ) {
		return "Cette banque est mal definie...";
		# Panic
	}

	conduse('FDN::Etebac::Etebac3');
	my $eth = open_prelevement({
		'emetteur'   => $cfg->{'emetteur'},
		'date'       => $pr->{'PRLBQ_DATE'},
		'raison-soc' => $cfg->{'raison-soc'},
		'dst' => {
			'banque'  => $cfg->{'banque'},
			'guichet' => $cfg->{'guichet'},,
			'compte'  => $cfg->{'compte'},
			'domiciliation' => $cfg->{'domicil'}
		}
	});
	for my $l ( @ls ) {
		print_prelevement($eth, {
			'nom' => $l->{'PRLLIG_TITULAIRE'}.'',
			'src' => {
				'banque' => $l->{'PRLLIG_BANQUE'},
				'guichet' => $l->{'PRLLIG_GUICHET'},
				'compte' => $l->{'PRLLIG_COMPTE'},
				'domiciliation' => $l->{'PRLLIG_DOMICIL'}
			},
			'libelle' => $l->{'PRLLIG_LIBELLE'},
			'montant' => $l->{'PRLLIG_MONTANT'},
			'emetteur' => $cfg->{'emetteur'}
		});
	}
	my $res = close_prelevement($eth,{
		'emetteur' => $cfg->{'emetteur'}
	});

	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','prldir');
	$dir = '/tmp' if ! defined $dir;
	my $file = $pr->{'PRLBQ_FICHIER'};
	$file = 'test.txt' if ! defined $file;
	if ( ! open OUTPUT, ">$dir/$file" ) {
		error "Failed to open file $dir/$file to write prl.";
		return undef;
	}
	print OUTPUT $res;
	close OUTPUT;
	set_prlbq_etat($pid,'envoye');
	return $res;
}

sub encaisser_facture($) { # Test
	my ( $fid ) = @_;
	my $facture = facture_par_id($fid);
	my @crs = creance_par_facture_id($fid);
	my $ttc = 0;
	my $ht  = 0;
	for my $cr ( @crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		set_creance_etat($crid,'payee');
		set_creance_datepayee($crid,'now');
		$ttc += $cr->{'CREANCE_TTC'};
		$ht  += $cr->{'CREANCE_HT'};
	}
	set_facture_etat($fid,'payee');
	set_facture_payee($fid,'now');
	if ( $ttc || $ht ) {
		new_tva($ttc-$ht,"Reglement facture ".$facture->{'FACTURE_NUM'});
	}
}

sub encaisser_prelevement($$$) { # Test
	my ( $pid, $payes, $rejets ) = @_;
	my $prlbq = prlbq_par_id($pid);
	my $ht  = 0;
	my $ttc = 0;
	for my $pl ( @$rejets ) {
		__changer_prllig($pl,'rejet','due');
	}
	for my $pl ( @$payes ) {
		my ( $h, $t ) = __changer_prllig($pl,'paye','payee');
		$ht += $h;
		$ttc += $t;
	}
	set_prlbq_etat($pid,'paye');
	new_tva($ttc-$ht, 'Prelevement '.$prlbq->{'PRLBQ_NOM'});
}

sub encaisser_paiement($$$$$) { # Test
	my ( $cid, $lib, $num, $ttc, $type ) = @_;
	my $pid = new_paiement($cid,$lib,$type,$num,$ttc);
	my $crid;
	my $tid;
	if ( $pid ) {
		$crid = new_creance($cid,-1,-$ttc,-$ttc,'0',"Paiement num. $pid");
	}
	return $pid;
}

sub changer_etat_paiement($$) { # Test
	my ( $pid, $etat ) = @_;
	my $p = paiement_par_id($pid);
	my $ttc = $p->{'PAIEMENT_TTC'};
	my $cid = $p->{'CLIENT_ID'};
	my $num = $p->{'PAIEMENT_NUM'};
	my $prev_etat = $p->{'PAIEMENT_ETAT'};
	if ( $prev_etat eq $etat ) {
		return undef;
	}
	my $crid;
	if ( $etat eq 'ko' ) {
		$crid = new_creance($cid,-1,$ttc,$ttc,'0',"Rejet paiement num. $pid");
		set_paiement_etat($pid,'ko');
	} elsif ( $etat eq 'ok' ) {
		$crid = new_creance($cid,-1,-$ttc,-$ttc,'0',"Reprise paiement num. $pid");
		set_paiement_etat($pid,'ok');
	} else {
		error "Le nouvel tat n'est pas valide";
		return undef;
	}
	return $crid;
}

sub rejeter_prllig($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prllig($pl,'rejet','due');
	new_tva($ht-$ttc,"Rej. prel. ".$pl->{'PRLLIG_LIBELLE'});
}

sub encaisser_prllig($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prllig($pl,'paye','payee');
	new_tva($ttc-$ht,"Prel. ligne ".$pl->{'PRLLIG_LIBELLE'});
}

sub __changer_prllig($$$) { # Test
	my ( $pl, $etat, $cretat ) = @_;
	__debug "Demande de changement de l'etat de la ligne ".$pl->{'PRLLIG_ID'}." pour passer a $cretat.";

	if ( $pl->{'PRLLIG_ETAT'} eq $etat ) {
		__debug "Deja dans cet etat.";
		return ( 0, 0);
	}
	my $ht  = 0;
	my $ttc = 0;
	my $plid = $pl->{'PRLLIG_ID'};
	my @crs = creance_par_prllig_id($plid);
	if ( $#crs == -1 ) {
		__debug "Pas de creance associee a la ligne de prelevement $plid.";
		return ( 0, 0 );
	}
	for my $cr ( @crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		__debug "Changement de la creance $crid.";
		set_creance_etat($crid,$cretat);
		$ttc += $cr->{'CREANCE_TTC'};
		$ht  += $cr->{'CREANCE_HT'};
	}
	__debug "Changement de la ligne $plid.";
	set_prllig_etat($plid,$etat);
	return ( $ht, $ttc );
}

sub if_ce_mois_ci($$$) { # Test
	my ( $date, $annee, $mois ) = @_;
	my $debut = $annee.'-'.$mois.'-01 00:00:00';
	my $fin   = $annee.'-'.$mois.'-31 00:00:00';
	if ( $date ge $debut && $date lt $fin ) {
		info 'Echeance ('.$date.') dans le mois (de '.$debut.' a '.$fin.')';
		return 1;
	} else {
		info 'Echeance ('.$date.') pas dans le mois (de '.$debut.' a '.$fin.')';
		return 0;
	}
}	       

sub if_fas($$$) { # Test
	my ( $ligne, $annee, $mois ) = @_;
	my $dtin = $ligne->{'LIGNE_DTFAC'};
	return if_ce_mois_ci($dtin, $annee, $mois);
}

#sub mois_prochain($) {
#	my ( $date ) = @_;
#	$date =~ /^(\d{4})-(\d{2})-(\d{2}) (\d\d):(\d\d):(\d\d)$/;
#	my ( $annee, $mois, $jour, $heure ) = ( $1, $2, $3, $4.$5.$6 );
#	$mois++;
#	if ( $mois == 13 ) {
#		$mois = 1;
#		$annee++;
#	}
#	while ( $jour >= $jours[$mois-1] ) {
#		$jour--;
#	}
#	$mois =~ s/^(.)$/0$1/;
#	$jour =~ s/^(.)$/0$1/;
#	return $annee.$mois.$jour.$heure;
#}
#
#sub annee_prochaine($) {
#	my ( $date ) = @_;
#	$date =~ /^(\d{4})-(\d{2})-(\d{2}) (\d\d):(\d\d):(\d\d)$/;
#	my ( $annee, $mois, $jour, $heure ) = ( $1, $2, $3, $4.$5.$6 );
#	$annee++;
#	return $annee.$mois.$jour.$heure;
#}

sub ligne_prorata($$$) { # Test
	my ( $ligne, $annee, $mois ) = @_;
	my $dtin = $ligne->{'LIGNE_DTFAC'};
	my $dtout = $ligne->{'LIGNE_DTOUT'};
	return __prorata($dtin,$dtout,$annee,$mois);
}

sub payant_abo($$$$$) { # Test
	my ($type,$debut,$fin,$annee,$mois) = @_;
	my $pr = __prorata($debut,$fin,$annee,$mois);
	if ( ! $pr ) {
		return FALSE;
	}
	if ( $type eq 'mensuel' ) {
		return TRUE;
	}
	$debut =~ /^(\d{4})-(\d{2})-/;
	my ( $anneed, $moisd ) = ($1,$2);
	if ( $moisd == $mois ) {
		return TRUE;
	}
}
#sub payant_prorata($$$) { # Test
#	my ( $debut, $fin, $annee, $mois ) = @_;
#	my $pr = __prorata($debut,$fin,$annee, $mois);
#	if ( $pr > .5 ) {
#		return 1;
#	} else {
#		return 0;
#	}
#}

sub __prorata($$$$) { # Test
	my ( $dtin, $dtout, $annee, $mois ) = @_;

	my $njours = $jours[$mois-1];
	my $debut  = $annee.'-'.$mois.'-01 00:00:00';
	my $fin    = $annee.'-'.$mois.'-31 00:00:00';

	if ( $dtin  eq '0000-00-00 00:00:00' ) {
		info "Prorata mois du $debut au $fin, debut non defini: 0 (pas encore a priori)";
		return 0;
	}
	if ( $dtout eq '0000-00-00 00:00:00' ) {
		$dtout = '2999-12-31 00:00:00';
	}
	if ( $dtin lt $debut && $dtout gt $fin ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 1 (plein)";
		return 1;
	}
	if ( $dtout lt $debut ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 0 (resilie)";
		return 0;
	}
	if ( $dtin gt $fin ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 0 (pas encore)";
		return 0;
	}
	my $ijour=0;
	my $ojour=$njours;															       
	if ( $dtin gt $debut && $dtout lt $fin ) {
		$dtin =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ijour = $1 - 1;
		$dtout =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ojour = $1;
	}
	if ( $dtin gt $debut ) {
		$dtin =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ijour = $1 - 1;
	}
	if ( $dtout lt $fin ) {
		$dtout =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ojour = $1;
	}
	my $prorata = ($ojour-$ijour)/$njours;
	info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: $prorata (partiel)";
	return $prorata;
}

1;
