package FDN::Adsl::Liste;

use strict;
use Exporter;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	liste_clients
	liste_clients_actifs
	liste_clients_resilies
	liste_lignes
	liste_unixgroups
	liste_validations
	liste_validations_new
	liste_validations_done
);
our @EXPORT_OK = qw (
);
our %EXPORT_TAGS = qw (
);

use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Web::List;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;

our $debug = 0;

$lists->{'clients'} = {
	'fct' => \&tous_clients,
	'title' => "Liste des abonns ".
		   forward_to('lclient-act','1',"").' '.
		   forward_to('lclient-resil','1',""),
	'sort' => {
		'id'  => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_ID'}     <=> $b->{'CLIENT_ID'} },
		'nom' => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_NOM'}    cmp $b->{'CLIENT_NOM'} },
		'adh' => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_NUMADH'} <=> $b->{'CLIENT_NUMADH'} },
		'adr' => sub ($$) { my ($a, $b) = @_; $a->{'ADRESSE_ID'}    <=> $b->{'ADRESSE_ID'} },
		'pai' => sub ($$) { my ($a, $b) = @_; $a->{'PRELEVEMT_ID'}  <=> $b->{'PRELEVEMT_ID'} },
		'eta' => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_ETAT'}   cmp $b->{'CLIENT_ETAT'} },
		'typ' => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_TYPE'}   cmp $b->{'CLIENT_TYPE'} }
	},
	'sort-default' => 'id',
	'rewrite-file' => 'adsl/rewrite.tbl',
	'rewrite-table' => 'client',
	'url' => 'liste-clients.cgi?x=y'
};

my $check;
BEGIN {
	$check = 1;
	if ( $debug ) {
		$check = $check && __check_list('clients') ;
	}
}

sub liste_clients(;$) {
	my ( $sort ) = @_;
	return __make_list('clients',$sort);
}

$lists->{'clients-act'} = {
	%{$lists->{'clients'}},
	'fct' => \&tous_clients_actifs,
	'title' => "Liste des abonns actifs ".
		   forward_to('lclient','0', '').' '.
		   forward_to('lclient-resil','1',"") ,
	'url' => 'liste-clients.cgi?act=1'
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('client-act');
	}
}

sub liste_clients_actifs(;$) {
	my ( $sort ) = @_;
	return __make_list('clients-act',$sort);
}

$lists->{'clients-resil'} = {
	%{$lists->{'clients'}},
	'fct' => \&tous_clients_resilies,
	'title' => "Liste des abonns rsilis ".
		   forward_to('lclient','0', '').''.
		   forward_to('lclient-act','1',"") ,
	'url' => 'liste-clients.cgi?resil=1',
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('client-resil');
	}
}

sub liste_clients_resilies(;$) {
	my ( $sort ) = @_;
	return __make_list('clients-resil',$sort);
}



$lists->{'lignes'} = {
	'fct' => \&toutes_lignes,
	'title' => "Liste des lignes ADSL",
	'url' => "liste-lignes.cgi?x=y",
	'sort' => {
		'id'   => sub ($$) { my ($a, $b) = @_; $a->{'LIGNE_ID'}     <=> $b->{'LIGNE_ID'} },
		'clid' => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_ID'}    <=> $b->{'CLIENT_ID'} },
		'tel'  => sub ($$) { my ($a, $b) = @_; $a->{'LIGNE_TEL'}    cmp $b->{'LIGNE_TEL'} },
		'etat' => sub ($$) { my ($a, $b) = @_; $a->{'LIGNE_STATE'}  cmp $b->{'LIGNE_STATE'} },
	},
	'sort-default' => 'id',
	'rewrite-file' => 'adsl/rewrite.tbl',
	'rewrite-table' => 'ligne'
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('lignes');
	}
}

sub liste_lignes(;$) {
	my ( $sort ) = @_;
	return __make_list('lignes',$sort);
}


$lists->{'validations'} = {
	'fct' => \&tous_validations,
	'title' => 'Liste des validations '.
		   forward_to('lval-new','1',"").' '.
		   forward_to('lval-done','1',""),
	'url' => 'liste-validations.cgi?x=y',
	'sort' => {
		'id'   => sub ($$) { my ($a, $b) = @_; $a->{'VALIDATION_ID'} <=> $b->{'VALIDATION_ID'} },
		'lid'  => sub ($$) { my ($a, $b) = @_; $a->{'LIGNE_ID'} <=> $b->{'LIGNE_ID'} },
		'cid'  => sub ($$) { my ($a, $b) = @_; $a->{'CLIENT_ID'} <=> $b->{'CLIENT_ID'} },
		'etat' => sub ($$) { my ($a, $b) = @_; $a->{'VALIDATION_ETAT'} cmp $b->{'VALIDATION_ETAT'} },
		'event'=> sub ($$) { my ($a, $b) = @_; $a->{'VALIDATION_EVENT'} cmp $b->{'VALIDATION_EVENT'} },
	},
	'sort-default' => 'id',
	'rewrite-file' => 'adsl/rewrite.tbl',
	'rewrite-table'=> 'validation'
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('validations');
	}
}

sub liste_validations(;$) {
	my ( $sort ) = @_;
	return __make_list('validations', $sort);
}

$lists->{'validations-new'} = {
	%{$lists->{'validations'}},
	'fct' => sub { return validation_par_etat('new'); },
	'title' => "Liste des validations en attente ".
		   forward_to('lval','1',"").' '.
		   forward_to('lval-done','1',""),
	'url' => 'liste-validations.cgi?new=1'
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('validations-new');
	}
}

sub liste_validations_new(;$) {
	my ( $sort ) = @_;
	return __make_list('validations-new', $sort);
}

$lists->{'validations-done'} = {
	%{$lists->{'validations'}},
	'fct' => sub { return validation_par_etat('done'); },
	'title' => "Liste des validations termines ".
		   forward_to('lval','1',"").' '.
		   forward_to('lval-new','1',""),
	'url' => 'liste-validations.cgi?done=1'
};

BEGIN {
	if ( $debug ) {
		$check = $check && __check_list('validations-done');
	}
}

sub liste_validations_done(;$) {
	my ( $sort ) = @_;
	return __make_list('validations-done', $sort);
}



$lists->{'unixgroups'} = {
	'fct' => \&tous_unixgroup,
	'title' => 'Liste des groupes unix '.
		   forward_to('nunixgroup',1,""),
	'url' => 'liste-unixgroup.cgi?x=y',
	'sort' => {
		'id'   => sub ($$) { my ($a, $b) = @_; $a->{'UNIXGROUP_ID'} <=> $b->{'UNIXGROUP_ID'} },
		'gid'  => sub ($$) { my ($a, $b) = @_; $a->{'UNIXGROUP_GID'} <=> $b->{'UNIXGROUP_GID'} },
		'name' => sub ($$) { my ($a, $b) = @_; $a->{'UNIXGROUP_NAME'} cmp $b->{'UNIXGROUP_NAME'} }
	},
	'sort-default' => 'id',
	'rewrite-file' => 'adsl/rewrite.tbl',
	'rewrite-table' => 'unixgroup'
};

BEGIN { 
	if ( $debug ) {
		$check = $check && __check_list('unixgroups');
	}
}

sub liste_unixgroups(;$) {
	my ( $sort ) = @_;
	return __make_list('unixgroups',$sort);
}




BEGIN {
	if ( $debug && ! $check ) {
		die "At least one list is not valid, plase check.";
	}
}

1;


