package FDN::Adsl::Map;

use strict;
use Exporter;
use FDN::Sql::Villes::Basic;
use FDN::Map::Base;
use FDN::Map::France;
use FDN::Adsl::Basic;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	map_dsp
	map_dsp_adh
	map_dsp_lig
	map_adherents
	map_adherents_dpt
	map_lignes
	map_lignes_dpt
	);
our @EXPORT_OK = qw ();


BEGIN {
	villes_select_db('adsl');

	new_color('lgreen',200,255,200);

	new_color('rose0',255,255,255);
	new_color('rose1',255,230,230);
	new_color('rose5',255,210,210);
	new_color('rose10',255,180,180);
	new_color('rose20',255,130,130);
	new_color('rose100',255,80,80);
	new_color('roseX',255,0,0);

	new_color('vert0',255,255,255);
	new_color('vert1',230,255,230);
	new_color('vert5',210,255,210);
	new_color('vert10',180,255,180);
	new_color('vert20',130,255,130);
	new_color('vert100',80,255,80);
	new_color('vertX',0,255,0);

	new_color('bleu0',255,255,255);
	new_color('bleu1',230,230,255);
	new_color('bleu5',210,210,255);
	new_color('bleu10',180,180,255);
	new_color('bleu20',130,130,255);
	new_color('bleu100',80,80,255);
	new_color('bleuX',0,0,255);
}

sub add_adr($$) {
	my ( $hash, $adr ) = @_;
	my $zip = $adr->{'ADRESSE_CP'};
	if ( $zip =~ /^75/ ) {
		$zip = '75000';
	}
	if ( $zip =~ /^1300/ ) {
		$zip = '13000';
	}
	if ( $zip =~ /^5400/ ) {
		$zip = '54100';
	}
	if ( $zip =~ /^3700/ ) {
		$zip = '37100';
	}
	if ( $zip =~ /^800[08]/ ) { # Amiens
		$zip = '80090';
	}
	if ( $zip =~ /^31[0-5]00$/ ) {
		$zip = '31500';
	}
	if ( $zip =~ /^06[0-3]00$/ ) {
		$zip = '06100';
	}
	if ( $zip == '80100' ) {
		$zip = '80132';
	}
	my $ville = $adr->{'ADRESSE_VILLE'};
	$ville =~ s/\s*cedex\s.*$//i;
	$ville = uc($ville);
	my $key = $zip.'/'.$ville;
	if ( ! defined $hash->{$key} ) {
		$hash->{$key} = {
			'count' => 0,
			'zip' => $zip,
			'ville' => $ville
		};
	}
	$hash->{$key}->{count}++;
}

sub map_draw_villes($$$$) {
	my ( $map, $villes, $circle_width, $color ) = @_;

	my $size;
	for my $k ( keys %$villes ) {
		$size = $circle_width *sqrt ($villes->{$k}->{count}) + 4;
		map_add_ville($map,$villes->{$k}->{zip},$villes->{$k}->{ville},'circle',$size,$color);
	}
}

sub map_draw_departements($$$) {
	my ( $map, $dpts, $basecol ) = @_;
	my $color;
	for my $k ( keys %$dpts ) {
		$color = '0';
		$color = '1' if ( $dpts->{$k} == 1 );
		$color = '5' if ( $dpts->{$k} > 1 );
		$color = '10' if ( $dpts->{$k} > 5 );
		$color = '20' if ( $dpts->{$k} > 10 );
		$color = '100' if ( $dpts->{$k} > 20 );
		$color = 'X' if ( $dpts->{$k} > 100 );
		map_add_dept($map,$k,$basecol.$color);
	}
}

sub map_get_adhs_villes(@) {
	my ( @adrs ) = @_;
	my $hash = {};
	for my $cl ( @adrs ) { 
		my $aid = $cl->{'ADRESSE_ID'};
		my $adr = adresse_par_id($aid);
		add_adr($hash,$adr);
	}
	return $hash;
}

sub map_get_adhs_depts(@) {
	my ( @adrs ) = @_;
	my $hash = {};
	for my $cl ( @adrs ) {
		my $aid = $cl->{'ADRESSE_ID'};
		my $adr = adresse_par_id($aid);
		my $dpt = $adr->{'ADRESSE_CP'};
		$dpt =~ s/^(\d\d).*$/$1/;
		$hash->{$dpt}++;
	}
	return $hash;
}

sub __map_dsp($) {
	my ( $map ) = @_;
	map_add_dept($map,'27','green'); #Eure
	map_add_dept($map,'42','lgreen'); #Loire
	map_add_dept($map,'45','lgreen'); #Loiret
	map_add_dept($map,'50','lgreen'); #Manche
	map_add_dept($map,'72','lgreen'); #Sarthe
	map_add_region($map,'LIMOUSIN', 'green');
	map_add_region($map,'ALSACE', 'lgreen');
}

sub __map_adherents($) {
	my ( $map ) = @_;
	my $circle_width = $map->{map}->{width}/250;
	my @cls = tous_clients_actifs();
	my $hash = map_get_adhs_villes(@cls);
	map_draw_villes($map,$hash,$circle_width,'red');
}

sub __map_lignes($) {
	my ( $map ) = @_;
	my $circle_width = $map->{map}->{width}/250;
	my @cls = ( ligne_par_state('ok'), ligne_par_state('building') );
	my $hash = map_get_adhs_villes(@cls);
	map_draw_villes($map,$hash,$circle_width,'blue');
}

sub map_dsp($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	__map_dsp($map);
	map_add_dept($map,undef,'light');
	map_draw_to($map,$file);
}

sub map_adherents($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	map_add_dept($map,undef,'light');
	__map_adherents($map);
	map_draw_to($map,$file);
}

sub map_lignes($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	map_add_dept($map,undef,'light');
	__map_lignes($map);
	map_draw_to($map,$file);
}

sub map_dsp_adh($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	__map_dsp($map);
	map_add_dept($map,undef,'light');
	__map_adherents($map);
	map_draw_to($map,$file);
}

sub map_dsp_lig($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	__map_dsp($map);
	map_add_dept($map,undef,'light');
	__map_lignes($map);
	map_draw_to($map,$file);
}

sub map_adherents_dpt($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	my @cls = tous_clients_actifs();
	my $hash = map_get_adhs_depts(@cls);
	map_draw_departements($map,$hash,'rose');
	map_add_dept($map,undef,'white');
	map_draw_to($map,$file);
	$map->{'map'}->save("greu.map");
}

sub map_lignes_dpt($$) {
	my ( $file, $xy ) = @_;
	my $map = new_map_france('PNG',$xy,$xy,'FDN',1);
	my @cls = ( ligne_par_state('ok'), ligne_par_state('building') );
	my $hash = map_get_adhs_depts(@cls);
	map_draw_departements($map,$hash,'bleu');
	map_add_dept($map,undef,'white');
	map_draw_to($map,$file);
	$map->{'map'}->save("greu.map");
}

1;
