package FDN::Adsl::OrdreTd;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		make_filename_9t
		make_filename_td
		make_filename_nerim
		make_filebody_9t
		make_filebody_td
		make_filebody_nerim
		mark_sent
		event_sent
		make_file_9t
		make_file_td
		make_file_nerim
		);
our @EXPORT_OK = qw ();


use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Web::TemplateParse;
use FDN::Adsl::Basic;
use POSIX qw(strftime);


sub make_filename_9t() { # Test
	my $fdnid = Config_Key('adsl/global.cfg','fdn','codeclient9');
	my $date = strftime "%Y%m%d", localtime;

	my $file = $fdnid.'_'.$date.'_';
	
	my $n = -1;
	my @ords;
	do {
		$n++;
		@ords = ordretd_par_fichier($file.$n."_CMD.xml");
	} while ( ! array_has_no_rows(\@ords) );

	if ( $n > 99 ) {
		return undef;
	}

	$file .= $n."_CMD.xml";
	return $file;
}

sub make_filename_td() { # Test
	my $fdnid = Config_Key('adsl/global.cfg','fdn','numclient');
	my $date = strftime "%Y%m%d", localtime;

	my $file = $fdnid.'_'.$date.'_';
	
	my $n = -1;
	my @ords;
	do {
		$n++;
		@ords = ordretd_par_fichier($file.$n.".agp");
	} while ( ! array_has_no_rows(\@ords) );

	if ( $n > 9 ) {
		return undef;
	}

	$file .= $n.".agp";
	return $file;
}

sub make_filename_nerim() { # Test
	my $date = strftime "%Y%m%d", localtime;
	my $file = 'fdn_'.$date.'_';

	my $n = -1;
	my @ords;

	do {
		$n++;
		@ords = ordretd_par_fichier($file.$n.".agp");
	} while ( ! array_has_no_rows(\@ords) );

	$file .= $n.'.csv';

	return $file;
}

sub make_filebody_9t(@) { # Test
	my @ordres = @_;
	
	my $fdnid = Config_Key('adsl/global.cfg','fdn','numclient9');
	my $date = strftime "%Y%m%d", localtime;
	my $hash = {
		'code_client' => $fdnid,
		'date_fichier' => $date,
		'commande' => []
	};
	for my $o ( @ordres ) {
		my $ligne = $o->{'ORDRETD_LIGNE'};
		push @{$hash->{'commande'}}, { 'ligne' => $ligne };
	}

	my $tpldir = Config_Key('adsl/global.cfg','dirs','soaptpl');
	my $file = remplace("$tpldir/cmds_9t.xml", $hash, undef);
	return $file;
}

sub make_filebody_td(@) { # Test
	my @ordres = @_;
	
	my $fdnid = Config_Key('adsl/global.cfg','fdn','numclient');
	my $date = strftime "%Y%m%d", localtime;
	my $head = $fdnid.';'.$date.';xDSLxx;@@2.1@@';
	my $crlf = "\r\n";

	my $file = $head.$crlf;
	for my $o ( @ordres )  {
		my $ordre = $o->{'ORDRETD_LIGNE'};
		$file .= $ordre.$crlf;
	}
	return $file;
}

sub make_filebody_nerim(@) { # Test
	my @ordres = @_;
	my $head = 'MASS_COMMAND;1.0;bureau@fdn.fr';
	my $crlf = "\n";
	my $file = $head.$crlf;
	for my $o ( @ordres ) {
		my $ordre = $o->{'ORDRETD_LIGNE'};
		$file .= $ordre.$crlf;
	}
	return $file;
}

sub mark_sent($@) {
	my ($file,@ordres) = @_;
	for my $o ( @ordres )  {
		my $oid = $o->{'ORDRETD_ID'};
		set_ordretd_etat($oid,'sent');
		set_ordretd_fichier($oid,$file);
	}
}

sub event_sent(@) {
	my @ordres = @_;
	conduse('FDN::Sql::Workflow::Basic');
	conduse('FDN::Sql::Workflow::Run');
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');

	for my $o ( @ordres ) {
		my $oid = $o->{'ORDRETD_ID'};
		send_event('ARFIC-'.$oid);
	}
}

sub __output_file($$$) { # Test
	my ( $basedir, $file, $body ) = @_;
	if ( ! open OUT, '>'.$basedir.'/'.$file ) {
		print STDERR "Echec a l'ouverture de $basedir/$file en ecriture.\n";
		return undef;
	}
	print OUT $body;
	close OUT;
	return $basedir.'/'.$file;
}

sub make_file_9t() { # Test
	my @ordres = ordretd_par_etat('new');
	if ( array_has_no_rows(\@ordres) ) {
		return undef;
	}
	my $basedir = Config_Key('adsl/global.cfg','dirs','9tout');
	my $file = make_filename_9t();
	if ( ! defined $file ) {
		return undef;
	}
	my $body = make_filebody_9t(@ordres);
	my $out = __output_file($basedir,$file,$body);
	if ( ! defined $out ) {
		return undef;
	}
	mark_sent($file,@ordres);
	return $out;
}


sub make_file_td() { # Test
	my @ordres = ordretd_par_etat('new');
	if ( array_has_no_rows(\@ordres) ) {
		return undef;
	}
	my $basedir = Config_Key('adsl/global.cfg','dirs','tdout');
	my $file = make_filename_td();
	if ( ! defined $file ) {
		return undef;
	}
	my $body = make_filebody_td(@ordres);
	my $out = __output_file($basedir,$file,$body);
	if ( ! defined $out ) {
		return undef;
	}
	mark_sent($file,@ordres);
	return $out;
}

sub make_file_nerim() { # Test
	my @ordres = ordretd_par_etat('new');
	if ( array_has_no_rows(\@ordres) ) {
		return undef;
	}
	my $basedir = Config_Key('adsl/global.cfg','dirs','nerimout');
	my $file = make_filename_nerim();
	if ( ! defined $file ) {
		return undef;
	}
	my $body = make_filebody_nerim(@ordres);
	my $out = __output_file($basedir,$file,$body);
	if ( ! defined $out ) {
		return undef;
	}
	mark_sent($file,@ordres);
	event_sent(@ordres);
	return $out;
}

1;

