package FDN::Adsl::Purge;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		purge_client
		);
our @EXPORT_OK = qw ();

use FDN::Adsl::Basic;
use FDN::Sql::Workflow::Basic;
use FDN::Sql::Workflow::Run;

BEGIN {
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');
}

sub purge_adresse($) {
	my ( $aid ) = @_;
	if ( $aid ) {
		del_adresse($aid);
	}
}

sub purge_deploiement($) {
	my ( $did ) = @_;
	if ( $did ) {
		del_deploiement($did);
	}
}

sub purge_prelevemt($) {
	my ( $pid ) = @_;

	if ( $pid ) {
		my $prl = prelevemt_par_id($pid);
		purge_adresse($prl->{'ADRESSE_ID'});
		del_prelevemt($pid);
	}
}

sub purge_ligne($) {
	my ( $lid ) = @_;
	if ( $lid ) {
		my $ligne = ligne_par_id($lid);
		purge_adresse($ligne->{'ADRESSE_ID'});
		my $depl = deploiement_par_ligne_id($lid);
		purge_deploiement($depl->{'DEPLOIEMENT_ID'});
		my $tid = $depl->{'TASK_ID'};
		my $task = task_par_id($tid);
		if ( defined $task ) {
			purge_task($depl->{'TASK_ID'});
		}
		del_ligne($lid);
	}
}

sub purge_client($) {
	my ( $cid ) = @_;

	my $client = client_par_id($cid);
	return if ( ref($client) ne 'HASH' );

	# 1. On purge son adresse
	purge_adresse($client->{'ADRESSE_ID'});

	# 2. On purge son prelevement
	purge_prelevemt($client->{'PRELEVEMT_ID'});

	# 3, On purge ses lignes
	my @ligs = ligne_par_client_id($cid);
	for my $ligne ( @ligs ) {
		purge_ligne($ligne->{'LIGNE_ID'});
	}

	del_client($cid);
}


1;
