package FDN::Adsl::Radius;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		create_radius_account
		destroy_radius_account
		change_radius_ip
		);
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Common::CharTable;
use FDN::Sql::Basic;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;

sub get_radius_login($$) { # Test
	my ( $nom, $groupe ) = @_;
	my $grp = radgroup_par_nom($groupe);
	if ( ! defined $grp ) {
		error "Nom de groupe invalide ($groupe)";
		return undef;
	}
	my $gid = $grp->{'RADGROUP_ID'};
	
	conduse('FDN::Common::CharTable');
	my $clean = cleanup_chars_as_table($nom, $iso8859_15_to_rfc822);
	$clean =~ s/^(.{,25}).*$/$1/;
	$clean =~ s/\s/./g;
	my $i = 0;
	my $login;
	if ( ! lock_radius() ) {
		error "Pas reussi a obtenir le lock sur LCK_RADIUS en 10 secondes, que faire...";
		return undef;
	}
	do {
		$login = $clean;
		if ( $i ) {
			$login .= '.'.$i;
		}
		$login .= '@fdn.nerim';
		$i++;
	} while ( raduser_par_login($login) );
	
	my $uid = new_raduser($login,$gid);
	unlock_radius();
	return $uid;
}

sub get_ip($$) { # Test
	my ( $uid, $type ) = @_;
	if ( ! lock_ip() ) {
		error "Pas reussi a obtenir le lock sur LCK_IP en 10 secondes, que faire...";
		return undef;
	}
	my $ip = ipaddr_par_etat_type('free',$type);
	if ( ! defined ( $ip ) ) {
		error "Il n'y a plus d'IP de type $type libre!";
		unlock_ip();
		return undef;
	}
	my $ipid = $ip->{'IPADDR_ID'};
	set_ipaddr_etat($ipid,'busy');
	set_ipaddr_raduser_id($ipid,$uid);
	unlock_ip();
	return $ipid;
}

sub free_ip($) { # Test
	my ( $ipid ) = @_;
	set_ipaddr_raduser_id($ipid,-1);
	set_ipaddr_etat($ipid,'free');
}

sub ip2num { unpack "N", pack "C4", @_ }

sub get_ipv6_route($) {
	my ( $ip ) = @_;
	my $ipv6_num = 0x1000 + ip2num(split(/\./, $ip)) - ip2num(80,67,176,0);
	return sprintf("2001:910:\%04x::/48", $ipv6_num);
}

sub create_radius_account($$;$) { # Test
	my ( $lid, $nom, $type ) = @_;
	if ( ! defined ( $type ) ) {
		$type = 'net1';
	}
	my $uid = get_radius_login($nom,'ADSL IP fixe');
	if ( ! defined ( $uid ) ) {
		error "Echec de creation d'un login radius pour $nom (ligne $lid)";
		return undef;
	}
	my $ipid = get_ip($uid, $type);
	if ( ! defined ( $ipid ) ) {
		error "Echec de reservation d'une IP pour $nom (ligne $lid)";
		del_raduser($uid);
		return undef;
	}
	my $ip = ipaddr_par_id($ipid);
	my $passwd = clef_aleatoire(8);

	new_uattr($uid,'check',
		'User-Password', '==', $passwd);
	new_uattr($uid,'reply',
		'Framed-IP-Address', '=', $ip->{'IPADDR_ADDR'});
	new_uattr($uid,'reply',
		'Framed-IP-Netmask', '=', $ip->{'IPADDR_MASK'});
	new_uattr($uid,'reply',
		'Framed-IPv6-Route', '=', get_ipv6_route($ip->{'IPADDR_ADDR'}));
	set_raduser_ligne_id($uid,$lid);
	return $uid;
}

sub change_radius_ip($;$) {
	my ( $uid, $type ) = @_;
	if ( ! defined ( $type ) ) {
		$type = 'net1';
	}
	my $ip = ipaddr_par_raduser_id($uid);
	my $ipid = get_ip($uid,$type);
	my $newip = ipaddr_par_id($ipid);
	my @uattrs = uattr_par_raduser_id($uid);
	if ( $#uattrs == -1 ) {
		error "On me demande de changer l'IP d'un abonne ($uid) qui n'a pas d'UATTR...";
		return;
	}
	for my $a ( @uattrs ) {
		my $aid = $a->{'UATTR_ID'};
		my $atr = $a->{'UATTR_ATTR'};
		if ( $atr eq 'Framed-IP-Address' ) {
			set_uattr_value($aid,$newip->{'IPADDR_ADDR'});
		} elsif ( $atr eq 'Framed-IP-Netmask' ) {
			set_uattr_value($aid,$newip->{'IPADDR_MASK'});
		} elsif ( $atr eq 'Framed-IPv6-Route' ) {
			set_uattr_value($aid,get_ipv6_route($newip->{'IPADDR_ADDR'}));
		}
	}
	free_ip($ip->{'IPADDR_ID'});
	return $ipid;
}

sub destroy_radius_account($) {
	my ( $uid ) = @_;
	my $rad = raduser_par_id($uid);
	my $ip = ipaddr_par_raduser_id($uid);
	my $login;
	if ( ! defined $rad ) {
		error "On me demande de detruire le compte radius $uid qui n'existe pas.";
		return;
	}
	if ( defined $ip ) {
		free_ip($ip->{'IPADDR_ID'});
	}
	$login = $rad->{'RADUSER_LOGIN'};
	$login =~ s/fdn.(fr|nerim)$/fdn\@INVALID/;
	set_raduser_login($uid,$login);
	return;
}


1;
