package FDN::Adsl::Sel;

use Exporter;
@ISA = ( 'Exporter' );
@EXPORT = qw(
	eligibilite
	
	make_soap
	get_reponse
	analyse_reponse
	);
@EXPORT_OK = qw(
	);

use strict;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::TemplateParse;
use HTTP::Headers;
use HTTP::Request;
use LWP::UserAgent;

our $debug = 1;

my $tbls = 'adsl/tables.cfg';
my $conf = 'adsl/elig/eliade.cfg';

sub __debug($) {
	my ( $msg ) = @_;
	debug $msg if ( $debug );
}

sub eligibilite($$) {
	my ( $tel, $offres ) = @_;
	my $soap = make_soap($tel,$offres);
	return undef if ! defined $soap;
	my $reponse = get_reponse($soap);
	return undef if ! defined $soap;
	my $res = analyse_reponse($tel,$reponse);
	return $res;
}

sub get_offre($) {
	my ( $offre ) = @_;
	my ( $debit, $option ) = @_;
	if ( $offre =~ /^(\d+(k|M|M2))(opt1|opt3)$/ ) {
		( $debit, $option ) = ($1,$3);
		$option =~ s/opt/option/;
	} else {
		error "Offre inconnue";
		return undef;
	}
	if ( $option !~ /^option(1|3)$/ ) {
		error "Option invalide";
		return undef;
	}
	my $res; 
	$res = Config_Key($tbls,$option,$debit);
	if ( ! defined $res ) {
		error "Offre inconnue";
		return undef;
	}
	$res = Config_Key($tbls,'numerical',$res);
	__debug "Offre trouvee, $offre se dit $res.";
	return $res; 
}

my %tbeq;

sub make_soap($$) {
	my ( $tel, $offres ) = @_;
	my $codes = [];
	for my $o ( @$offres ) {
		my $c = get_offre($o);
		push @$codes, { 'CODE' => $c} if defined $c;
		$tbeq{$c} = $o if defined $c;
	}
	my $template = Config_Key($conf,'soap','template');
	my $methode  = Config_Key($conf,'soap','methode');
	my $urn      = Config_Key($conf,'soap','urn');
	__debug "Template utilise: $template.";
	__debug "Methode a appeler: $methode.";
	__debug "Urn SOAP: $urn.";
	my $infos = {
		"METHOD" => $methode,
		"URN"    => $urn,
		"TEL"    => $tel,
		"ETOILE" => 3,
		"CLIINFO" => "c'est optionnel mais ca ne fonctionne pas sans",
		"offres" => $codes,
		"PROTO"  => '1.0'
	};
	my $soap = remplace($template,$infos);
	__debug "Ca donne ca comme requete: $soap";
	return $soap;
}

sub get_reponse($) {
	my ( $soap ) = @_;
	my $protocol = Config_Key($conf,'eliade','protocol');
	my $login    = Config_Key($conf,'eliade','login');
	my $pass     = Config_Key($conf,'eliade','pass');
	my $url      = Config_Key($conf,'eliade','url');
	my $cgi      = Config_Key($conf,'eliade','cgi');
	my $urn      = Config_Key($conf,'soap','urn');
	my $methode  = Config_Key($conf,'soap','methode');
	$url = $protocol."://".$login.":".$pass."@".$url.$cgi;
	__debug "URL complete: $url";
	my $header = new HTTP::Headers;
	$header->header('Content-Type' => 'text/xml; charset=utf-8');
	$header->header('SOAPAction' => $urn."/".$methode);
	my $request = new HTTP::Request ('POST', $url, $header );
	$request->content($soap);
	my $ua = LWP::UserAgent->new;
	$ua->timeout(60);
	my $response = $ua->request($request);
	if ( $response->message =~ /timeout/ ) {
		error "Le serveur TD n'a pas repondu assez vite.";
		return undef;
	}
	__debug "Reponse HTTP: ".$response->message;
	__debug "Reponse SOAP: ".$response->content;
	return $response->content;
}

sub analyse_reponse($$) {
	my ( $tel, $reponse ) = @_;
	my $infos = split_reponse($tel,$reponse);
	if ( $infos->{'etat'} ) {
		return $infos;
	}
	my $res = {};
	for my $rep ( @{$infos->{'offres'}} ) {
		my $offre = {};
		
		my $num = readString('offre',$rep);
		if ( ! defined $tbeq{$num} ) {
			$infos->{'etat'} = 1;
			push @{$infos->{'message'}}, 'TD repond pour des offres qu\'on n\'a par demande.';
		}
		$res->{$tbeq{$num}} = $offre;
		$offre->{'offre'} = $tbeq{$num};
		
		my $faisabilite = readString('faisabilite',$rep);
		my $fais = Config_Key($tbls,'faisable',$faisabilite);
		if ( ! defined $fais ) {
			$infos->{'etat'} = 1;
			push @{$infos->{'message'}}, 'Code faisabilite invalide.';
			$fais = 0;
		}
		
		my $motifs = readString('motifs',$rep);
		my $alertes = readString('alertes',$rep);
		$motifs = readStrings('string',$motifs);
		$alertes = readStrings('string',$alertes);

		for my $a ( @$alertes ) {
			if ( $a eq 'A1' ) {
				$infos->{'etat'} = 3;
				push @{$infos->{'message'}}, 'Ligne numeris';
				next;
			}
			if ( $a eq 'A4' ) {
				my $modem = readString('modem',$rep);
				$modem = readString('string',$modem);
				$offre->{'modem'} = $modem;
				next;
			}
			if ( $a eq 'A5' ) {
				if ( $fais != 2 ) {
					push @{$infos->{'message'}}, 'TD donne une date previsionnelle pour une ligne deja dispo.';
				}
				$offre->{'date'} = readString('dateOuverture',$rep);
				next;
			}
			if ( $a eq 'A8' ) {
				$offre->{'infos'} .= "Il faut un modem bipaire.\n";
				next;
			}
			if ( $a eq 'LR' ) {
				$offre->{'infos'} .= "Liaison de raccordement en multi-NRA.\n";
				next;
			}
			push @{$infos->{'message'}}, 'TD emet des alertes non definies';
		}
		for my $m ( @$motifs ) {
			my $good = Config_Key($tbls,'faisable-motifs',$m);
			$offre->{'infos'} .= "$m:\n".Config_Key($tbls,'motifs',$m)."\n";
			if ( $fais == 3 && ! $good ) {
				$offre->{'infos'} .= "Motif serieux. Pas faisable.\n";
				$fais = 0;
			}
		}
		if ( $fais == 2 ) {
			$offre->{'infos'} .= "TD indique que ce n'est pas faisable maintenant.";
			$fais = 0;
		}
		if ( $fais ) {
			$offre->{'faisable'} = 1;
		} else {
			$offre->{'faisable'} = 0;
		}
	}
	$infos->{'offres'} = $res;
	return $infos;
}

sub split_reponse($$) {
	my ( $tel, $reponse ) = @_;
	my $res = {};
	$res->{'message'} = [];
	
	my $body = readString('getEligibiliteResult',$reponse);
	if (! defined $body ) {
		error "Le serveur eliade n'a pas repondu";
		$res->{'etat'} = 1;
		$res->{'messsage'} = 'Le serveur TD repond n\'importe quoi';
		return $res;
	}
	$res->{'body'} = $body;
	
	my $retour = readString('codeRetour',$body);
	if (    ! defined $retour ||
		$retour !~ /^-?\d+$/ ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD repond n\'importe quoi';
		return $res;
	}
	if ( $retour == 1 ) {
		$res->{'etat'} = 2;
		push @{$res->{'message'}}, 'Requete invalide (bug FDN)';
		return $res;
	}
	if ( $retour == 2 ) {
		$res->{'etat'} = 2;
		push @{$res->{'message'}}, 'Requete incomplete (bug FDN)';
		return $res;
	}
	if ( $retour == 11 ) {
		$res->{'etat'} = 3;
		push @{$res->{'message'}}, 'Numero de telephone non attribue';
		return $res;
	}
	if ( $retour == 12 ) {
		$res->{'etat'} = 3;
		push @{$res->{'message'}}, 'Numero de telephone inconnu';
		return $res;
	}
	if ( $retour == 13 ) {
		$res->{'etat'} = 3;
		push @{$res->{'message'}}, 'Cette ligne fait partie d\'un groupement de lignes';
		return $res;
	}
	if ( $retour == -1 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur FT ne repond pas (panne)';
		return $res;
	}
	if ( $retour == -2 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD est en dehors de ses heures de travail';
		return $res;
	}
	if ( $retour == -3 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD est surcharge';
		return $res;
	}
	if ( $retour == -4 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD trouve qu\'on parle trop vide';
		return $res;
	}
	if ( $retour != 0 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD repond des codes non-prevus. Doit etre malade.';
		return $res;
	}
	$res->{'etat'} = 0;
	
	my $nd = readString('nd',$body);
	if ( $nd ne $tel ) {
		error "Le serveur eliade n'a pas repondu pour la bonne ligne";
		$res->{'etat'} = 1;
		$res->{'message'} = 'Le serveur TD repond sur le mauvais numero de telephone';
		return $res;
	}
	$res->{'nd'} = $nd;
	
	my $insee = readString('codeInsee',$body);
	$res->{'insee'} = $insee;
	
	my $repar = readString('codeRepartiteur',$body);
	$res->{'repartiteur'} = $repar;
	
	my $info = readString('infoClient',$body);
	$res->{'info'} = $info;
	
	my $offres = readString('offres',$body);
	if ( ! defined $offres ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD fait des reponses vides.';
		return $res;
	}
	$offres = readStrings('ResponseOffre',$offres);
	if ( $#$offres == -1 ) {
		$res->{'etat'} = 1;
		push @{$res->{'message'}}, 'Le serveur TD fait des reponses vides.';
		return $res;
	}
	$res->{'offres'} = $offres;
	return $res;
}

sub readString($$) {
	my ( $tag, $text ) = @_;
	$text =~ /<$tag>(.*)<\/$tag>/;
	return $1;
}

sub readStrings($$) {
	my ( $tag, $text ) = @_;
	my @res;
	if ( ! defined $tag || !defined $text ) {
		return [];
	}
	while ($text =~ /<$tag>(.*?)<\/$tag>/g) {
		push @res, $1;
	}
	return \@res;
}

1;
