#!/usr/bin/perl

use strict;
use FDN::Common::Config;
use FDN::Web::TemplateParse;

my $link_subs = Config_Key('adsl/global.cfg','web','link_subs');
my $link_adhs = Config_Key('adsl/global.cfg','web','link_adhs');

my $menu = {
menus => [
{ 'nom' => 'Documentation',
  'entries' => [
  	{ 'lien' => 'docs.cgi?page=index.html', 'nom' => 'Documentation' },
  	] },
{ 'nom' => 'Outils techniques FDN',
  'entries' => [
	{ 'lien' => 'liste-lignes.cgi',   'nom' => 'Lister les lignes' },
	{ 'lien' => 'view-ligne.cgi',     'nom' => 'Voir une ligne' },
	{ 'lien' => 'liste-clients.cgi',  'nom' => 'Lister les clients'},
	{ 'lien' => 'view-client.cgi',    'nom' => 'Voir un client' },
	{ 'lien' => 'liste-ipaddr.cgi',   'nom' => 'Lister les adresses IP'},
	{ 'lien' => 'view-ipaddr.cgi',    'nom' => 'Voir une adresse IP'},

	{ 'lien' => 'liste-prix.cgi',     'nom' => 'Grille des tarifs'},
	{ 'lien' => 'new-scan.cgi?dpl=1', 'nom' => 'Rception de documents (souscription)'},
	{ 'lien' => 'new-scan.cgi',       'nom' => 'Rception de documents (hors souscription)'},
	{ 'lien' => 'paye-fas.cgi',       'nom' => 'Validation des FAS d\'une ligne'},
	{ 'lien' => 'liste-validations.cgi','nom' => 'Validation de la perte d\'une ligne'},
	{ 'lien' => 'new-ipaddr.cgi',     'nom' => 'Ajouter des adresses IP publiques'},
	{ 'lien' => 'resil-ligne.cgi',    'nom' => 'Rsilier une ligne ADSL'},
	{ 'lien' => 'draw-maps.cgi',      'nom' => 'Refaire les cartes'},
	] },
{ 'nom' => 'Outils debug FDN',
  'entries' => [
	{ 'lien' => 'new-prix.cgi',       'nom' => 'Crer un nouveau tarif'},
  	{ 'lien' => 'new-creance.cgi',	  'nom' => 'Crer une crance'},
	{ 'lien' => 'new-prelevemt.cgi',  'nom' => 'Crer un compte en banque'},
	{ 'lien' => 'new-paiement.cgi',   'nom' => 'Crer un paiement'},
	{ 'lien' => 'liste-waiting.cgi',  'nom' => 'Voir les tches en attente d\'un vennement'},
	{ 'lien' => 'new-run.cgi',	  'nom' => 'Relancer une tache'},
	{ 'lien' => 'new-event.cgi',      'nom' => 'Envoyer un vennement arbitraire'},
	{ 'lien' => 'unlock-depl.cgi',    'nom' => 'Dploiements bloqus'},
	{ 'lien' => 'view-raduser.cgi',   'nom' => 'Voir un compte radius'},
	{ 'lien' => 'liste-task.cgi',     'nom' => 'Liste des tches de dploiement'},
	] },
{ 'nom' => 'Liens inutilisables tel quel (utilises par les autres)',
  'entries' => [
	{ 'lien' => 'new-client.cgi',     'nom' => '(NOUSE) Crer un nouveau client'},
	{ 'lien' => 'view-task.cgi',      'nom' => '(NOUSE) Voir les dtails sur une tche'},
	{ 'lien' => 'view-adresse.cgi',   'nom' => '(NOUSE) Voir une adresse'},
	{ 'lien' => 'view-prelevemt.cgi', 'nom' => '(NOUSE) Voir un prelevement'},
	{ 'lien' => 'view-creance.cgi',   'nom' => '(NOUSE) Voir une creance'},
	{ 'lien' => 'view-payant.cgi',    'nom' => '(NOUSE) Voir un payant'},
	{ 'lien' => 'view-paiement.cgi',  'nom' => '(NOUSE) Voir un paiement'},
	{ 'lien' => 'view-image.cgi',     'nom' => '(NOUSE) Voir une image'},
	{ 'lien' => 'view-info.cgi',      'nom' => '(NOUSE) Voir une info'},
	{ 'lien' => 'view-prix.cgi',      'nom' => '(NOUSE) Voir un prix'},
	{ 'lien' => 'view-prlbq.cgi',     'nom' => '(NOUSE) Voir un prlvement'},
	{ 'lien' => 'view-prllig.cgi',    'nom' => '(NOUSE) Voir une ligne de prlvement'},
	{ 'lien' => 'view-facture.cgi',   'nom' => '(NOUSE) Voir une facture'},
	] },
{ 'nom' => 'Outils compta/factu',
  'entries' => [
  	{ 'lien' => 'liste-clotures.cgi', 'nom' => 'Lister les clotures mensuelles'},
	{ 'lien' => 'clore-mois.cgi',     'nom' => 'Clore un mois'},
	{ 'lien' => 'liste-creances.cgi', 'nom' => 'Lister les crances dues'},
	{ 'lien' => 'liste-factures.cgi', 'nom' => 'Lister les factures'},
	{ 'lien' => 'crea-facture1.cgi',  'nom' => 'Crer une facture'},
	{ 'lien' => 'liste-prlbq.cgi',    'nom' => 'Lister les prlvements'},
	{ 'lien' => 'new-prlbq.cgi',      'nom' => 'Crer un prlvement'},
	{ 'lien' => 'print-facture.cgi',  'nom' => 'Imprimer une facture'},
	{ 'lien' => 'print-prlbq.cgi',    'nom' => 'Imprimer un prlvement'},
	{ 'lien' => 'print-prllig.cgi',   'nom' => 'Imprimer une ligne de prlvement'},
	{ 'lien' => 'envoie-facture.cgi', 'nom' => 'Envoyer une facture'},
	{ 'lien' => 'envoie-preleve.cgi', 'nom' => 'Envoyer un prlvement'},
	{ 'lien' => 'paye-facture.cgi',   'nom' => 'Encaisser une facture'},
	{ 'lien' => 'paye-preleve.cgi',   'nom' => 'Encaisser un prlvement'},
	{ 'lien' => 'liste-tvadcl.cgi',   'nom' => 'Dclarations de TVA prcdentes'},
	{ 'lien' => 'voir-tva.cgi',       'nom' => 'Voir la TVA en attente'},
	{ 'lien' => 'envoie-tva.cgi',     'nom' => 'Dclarer la TVA en attente'},
	{ 'lien' => 'paye-tva.cgi',       'nom' => 'Dcaisser la TVA'},
	{ 'lien' => 'liste-compta.cgi',   'nom' => 'Voir un document FDN'},
	{ 'lien' => 'new-compta.cgi',     'nom' => 'Crer une nouvelle catgorie de documents FDN'},
	] },
{ 'nom' => 'Outils publics/adhrents/abonns',
  'entries' => [
	{ 'lien' => $link_subs,           'nom' => 'Sites de souscription'},
	{ 'lien' => $link_adhs,           'nom' => 'Tableau de bord des adhrents'},
	] },
] };

my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');

my $page = remplace($dir.'/menu.tpl',$menu,1);
print "Content-Type: text/html; charset=ISO-8859-1\n\n";
print remplace($dir.'/base.tpl',{ 'PAGE' => $page },1);

