#!/usr/bin/perl

use strict;
use CGI;
use FDN::Web::Forms;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;

my $cgi = new CGI;
my $vars = $cgi->Vars;

my $form = form_read('adsl/form/new.cfg','nipaddr');
if ( $vars->{'do'} ne 'yes' ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}
if ( !check_transition($form,$vars) ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}

# Controles sur les champs

my $invalid = 0;

my $debut = $vars->{'ipstart'};
my $fin   = $vars->{'ipstop'};

$debut =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
my ( $d1, $d2, $d3, $d4 ) = ( $1, $2, $3, $4 );
my $dbin = ( $1 << 24 ) + ( $2 << 16 ) + ( $3 << 8 ) + $4;

$fin =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
my ( $f1, $f2, $f3, $f4 ) = ( $1, $2, $3, $4 );
my $fbin = ( $1 << 24 ) + ( $2 << 16 ) + ( $3 << 8 ) + $4;

if ( $d1 < 0 || $d1 > 255 ||
     $d2 < 0 || $d2 > 255 ||
     $d3 < 0 || $d3 > 255 ||
     $d4 < 0 || $d4 > 255 ||
     $f1 < 0 || $f1 > 255 ||
     $f2 < 0 || $f2 > 255 ||
     $f3 < 0 || $f3 > 255 ||
     $f4 < 0 || $f4 > 255 ) {
	add_form_error('syntax',"C'est pas une IP ca.");
	$invalid = 1;
}

my $type = $vars->{'iptype'};
my $size = $type;
$size =~ s/^\D+(\d+)$/$1/;

if ( $dbin % $size ) {
	add_form_error('syntax',"L'IP de depart ne cadre pas avec la taille de subnet");
	$invalid = 1;
}
if ( ($fbin+1) % $size ) {
	add_form_error('syntax',"L'IP de fin ne cadre pas avec la taille du subnet");
	$invalid = 1;
}

if ( $dbin > $fbin ) {
	add_form_error('syntax',"L'IP de depart est apres l'IP de fin");
	$invalid = 1;
}


# Controle de l'existence des IPs

my $mask_type = {
	'net1' => '255.255.255.255',
	'net8' => '255.255.255.248'
};

my $ip = $dbin;
my $page;
my $n;
my ( $i1, $i2, $i3, $i4, $mask, $quad );
$mask = $mask_type->{$type};
while ( $ip <= $fbin ) {
	$i4 =  $ip        % 256;
	$i3 = ($ip >> 8)  % 256;
	$i2 = ($ip >> 16) % 256;
	$i1 = ($ip >> 24) % 256;
	$quad = "$i1.$i2.$i3.$i4";
	#$page .= "<p>Creation de $quad/$mask type $type etat 'free'.</p>";
	my $previp = ipaddr_par_addr($quad);
	if ( $previp ) {
		add_form_error('syntax',"L'ip $quad existe deja dans la base");
		$invalid = 1;
	}
	$ip += $size;
}

if ( $invalid ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}

# Creation effective des nouvelles IP
$ip = $dbin;
while ( $ip <= $fbin ) {
	$i4 =  $ip        % 256;
	$i3 = ($ip >> 8)  % 256;
	$i2 = ($ip >> 16) % 256;
	$i1 = ($ip >> 24) % 256;
	$quad = "$i1.$i2.$i3.$i4";
	my $ipid = new_ipaddr($quad,$mask,$type);
	if ( ! defined $ipid ) {
		$page .= "<div class=\"error\"><p>Echec de la creation de l'IP $quad</p></div>\n";
	} else {
		$n++;
	}
	$ip += $size;
}
$page .= "<p>Il a ete cree $n nouvelles entrees.</p>";

eject_page($page);

#my $index = Config_Key('global-adsl.cfg','web','index');
#exec($index);
