#!/usr/bin/perl

use strict;
use CGI;
use FDN::Common::Config;
use FDN::Common::Mail;
use FDN::Web::Forms;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;

my $cgi = new CGI;
my $vars = $cgi->Vars;


my $form = form_read('adsl/form/new.cfg','nscan');
ajoute_liste_scanners($form);
ajoute_liste_comptas($form);
ajoute_liste_clients($form);
if ( defined $vars->{'dpl'} ) {
	ajoute_liste_deploiement($form);
	ajoute_champ_config($form,'adsl/form/new.cfg','hdo','hdpl');
} else {
	ajoute_liste_lignes($form);
}
if ( $vars->{'do'} ne 'yes' ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}
if ( !check_transition($form,$vars) ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}

my $scan = $vars->{'scanner'};
my $doctype = $vars->{'doctype'};
my $resol = $vars->{'resolution'};
my $notif = $vars->{'notif'};
my $page  = $vars->{'page'};
my $titre;
my $val = 'img';
my $niveau;
if ( $doctype eq 'test' ) {
	$titre  = 'test:';
	$val    = 'test';
	$niveau = 'none';
} elsif ( $doctype eq 'mandat' ) {
	$titre  = 'img:Mandat de degroupage';
	$niveau = 'ligne';
} elsif ( $doctype eq 'adhesion' ) {
	$titre  = 'img:Bulletin d\'adhesion';
	$niveau = 'client';
} elsif ( $doctype eq 'abo1' ) {
	$titre  = 'img:Bulletin d\'abonnement page 1/2';
	$niveau = 'ligne';
} elsif ( $doctype eq 'abo2' ) {
	$titre  = 'img:Bulletin d\'abonnement page 2/2';
	$niveau = 'ligne';
} elsif ( $doctype eq 'prele' ) { 
	$titre  = 'img:Autorisation de prelevement';
	$niveau = 'client';
} elsif ( $doctype eq 'ft' ) {
	$titre  = 'img:Facture France Telecom';
	$niveau = 'ligne';
} elsif ( $doctype eq 'id' ) {
	$titre  = 'img:Carte d\'identite';
	$niveau = 'client';
} elsif ( $doctype eq 'etudiant' ) {
	$titre  = 'img:Carte d\'etudiant ou assimile';
	$niveau = 'client';
} elsif ( $doctype eq 'ligne' ) {
	$titre  = 'img:';
	my $tmp = $vars->{'titre-ligne'};
	$tmp = 'Document sans titre' if ! defined $tmp;
	$titre .= $tmp;
	$niveau = 'ligne';
} elsif ( $doctype eq 'client' ) {
	$titre  = 'img:';
	my $tmp = $vars->{'titre-client'};
	$tmp = 'Document sans titre' if ! defined $tmp;
	$titre .= $tmp;
	$niveau = 'client';
} elsif ( $doctype eq 'compta' ) {
	$titre  = 'img:';
	my $tmp = $vars->{'titre-compta'};
	$tmp = 'Document sans titre' if ! defined $tmp;
	$titre .= $tmp;
	$niveau = 'compta';
}

if ( defined $page ) {
	if ( $page =~ /^(\d+)\/(\d+)$/ ) {
		my ( $num, $tot ) = ( $1, $2 );
		$titre .= " page $page";
		if ( $num < $tot ) {
			$notif = 'personne';
		}
	}
}
my $iid = new_info('fichier',$titre,'tmp');
$val .= $iid;
set_info_val($iid,$val);

my $cid = $vars->{'client'};
my $lid = $vars->{'ligne'};
my $coid = $vars->{'compta'};

my $ret = get_image($scan,$val,$resol);
if ( ! $ret ) {
	form_set_field_value($form->{'byname'}->{'infoscan'},
		'<p>L\'image n\'a pas pu tre numrise.</p>');
	eject_page(affiche_form($form,$vars));
	exit(0);
}

if ( $niveau eq 'ligne' ) {
	new_ligneinfo($lid,$iid);
	if ( $doctype eq 'mandat' ) {
		my $dpl = deploiement_par_ligne_id($lid);
		my $did = $dpl->{'DEPLOIEMENT_ID'};
		my $mdt = mandat_par_deploiement_id($did);
		my $mid = $mdt->{'MANDAT_ID'};
		set_mandat_image($mid,$val);
	}
} elsif ( $niveau eq 'client' ) {
	new_clientinfo($cid,$iid);
} elsif ( $niveau eq 'compta' ) {
	new_comptainfo($coid,$iid);
}

form_set_field_value($form->{'byname'}->{'infoscan'},
	'<p>L\'image a t numrise sous le nom '.$val.' et devrait tre disponible '.
	'sous peu <a href="view-image.cgi?name='.$val.'">ici</a>.</p>');

my $notif_adr;
if ( $notif eq 'tresorier' ) {
	$notif_adr = 'tresorier@fdn.fr';
} elsif ( $notif eq 'secretaire' ) {
	$notif_adr = 'secretaire@fdn.fr';
} elsif ( $notif eq 'president' ) {
	$notif_adr = 'president@fdn.fr';
} elsif ( $notif eq 'bureau' ) {
	$notif_adr = 'bureau@fdn.fr';
}

if ( $notif_adr ) {
	my $infos = {
		'scanner' => $vars->{'scanner'},
		'titre'   => $titre,
		'info_id' => $iid,
		'page'    => $page
	};
	if ( ! set_mail_config('/etc/fdn/adsl/mail.cfg') ) {
		form_set_field_value($form->{'byname'}->{'infoscan'},
			'<p>Le mail de notification n\'a pas pu tre envoy, je n\'ai pas trouv mon fichier de configuration.</p>');
	}
	if ( ! envoie_mail($notif_adr, 'notifscan', $infos) ) {
		form_set_field_value($form->{'byname'}->{'infoscan'},
			'<p>Le mail de notification n\'a pas pu tre envoy.</p>');
	}
}

eject_page(affiche_form($form,$vars));
exit(0);


sub get_image($$$) {
	my ( $host, $img, $resol ) = @_;
	my $cfg = Config_Section('adsl/global.cfg','scan-'.$host);
	return if ( !defined($cfg) );
	return if ( $img !~ /^[a-z]+[0-9]+$/ );
	my $remote = $cfg->{'host'};
	my $prog = $cfg->{'prg'};
	my $port = $cfg->{'port'};
	if ( open (REMOTE,'-|','ssh','-p',"$port","$remote","$prog $img $resol") ) {
		my $line = <REMOTE>;
		if ( $line =~ /^OK\s*$/ ) {
			return 1;
		}
	}
	return 0;
}


