#!/usr/bin/perl

use strict;
use CGI;
#use FDN::Config;
use FDN::Common::Logger;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Sql::Workflow::Basic;
use FDN::Adsl::Utils;
#use FDN::Workflow::Wf;
use POSIX qw(strftime);
use Data::Dumper;

workflow_select_db('adsl');
my $cgi = new CGI;
my $vars = $cgi->Vars;

sub remove_failed_subtask($$$) {
	my ( $tid, $props, $key ) = @_;
	
	my $failed = task_par_key($key);
	my $fid = $failed->{'TASK_ID'};
	if ( ! defined $props->{'ERRs'} ) {
		$props->{'ERRs'} = [];
	}
	
	my $err = {};
	$err->{'date'} = strftime "%Y/%m/%d %H:%M:%S", localtime;
	$err->{'descr'} = "Tache $key en echec.";
	$err->{'debut'} = $failed->{'TASK_DTIN'};
	$err->{'fin'}   = $failed->{'TASK_DTMOD'};
	$err->{'task-id'} = $fid;
	push @{$props->{'ERRs'}}, $err;
		
	set_task_key($fid,'failure-'.$fid);
	del_taskself($tid,$fid);
	new_taskself($tid,$fid,-1);
	my @ws = waiting_par_task($tid);
	for my $wait ( @ws ) {
		$err = {};
		if ( $wait->{'WAITING_EVENT'} eq 'EOT-'.$key ) {
			$err->{'date'} = strftime "%Y/%m/%d %H:%M:%S", localtime;
			$err->{'descr'} = "Ne plus attendre la tache $key en echec";
			push @{$props->{'ERRs'}}, $err;
			del_waiting($wait->{'WAITING_ID'});
		}
	}
}

sub remove_done_subtask($$$) {
	my ( $tid, $props, $key ) = @_;
	
	my $failed = task_par_key($key);
	my $fid = $failed->{'TASK_ID'};
	if ( ! defined $props->{'ERRs'} ) {
		$props->{'ERRs'} = [];
	}
	
	my $err = {};
	$err->{'date'}    = strftime "%Y/%m/%d %H:%M:%S", localtime;
	$err->{'descr'}   = "Annulation tache $key pour reprendre suite a un echec ulterieur";
	$err->{'debut'}   = $failed->{'TASK_DTIN'};
	$err->{'fin'}     = $failed->{'TASK_DTMOD'};
	$err->{'task-id'} = $fid;
	push @{$props->{'ERRs'}}, $err;
		
	del_event_par_event('EOT-'.$key);
	set_task_key($fid,'failure-'.$fid);
	del_taskself($tid,$fid);
	new_taskself($tid,$fid,-1);
}

my $page;
if ( defined $vars->{'lock'} ) {
	my $tid = $vars->{'tid'};
	my $lock = $vars->{'lock'};
	if ( $lock eq 'noevent' ) {
		# Pas d'autre evennement attendu
		# On regarde quel est le dernier fils connu termine
		my @tss = taskself_par_parent($tid);
		if ( array_has_no_rows(\@tss) ) {
			# Envoyer init
			new_run($tid,'init');
		} else {
			# Envoyer la fin du dernier fils termine
			my $lastson = -1;
			my $order = -1;
			for my $t ( @tss ) {
				if ( $t->{'TASKSELF_ORDER'} > $order ) {
					$order = $t->{'TASKSELF_ORDER'};
					$lastson = $t->{'SELF_ID'};
				}
			}
			my $son = task_par_id($lastson);
			new_run($tid,'FEOT-'.$son->{'TASK_KEY'});
		}
	} elsif ( $lock eq 'endofphantom' ) {
		error "La tache attend la fin d'une sous-tache qui n'a ".
		      "pas reussi a se lancer. Je ne sais pas traiter ce ".
		      "cas la.";
		# La tache attend la fin d'une sous-tache qui n'a pas reussi
		# a se lancer
	} elsif ( $lock =~ /at E\/(\d+) on E\/\1 failed/ ) {
		my $fix = $1;
		my $elkey = 'E/'.$fix;
		info "La tache est bloquee sur un echec d'eligibilite. On suppose que ".
		     "les modifications necessaires on ete apportees sur le demande. ".
		     "Il faut juste relancer la tche sur init.";
		my $props = get_task_properties($tid);
		remove_failed_subtask($tid,$props,$elkey);
		update_task_properties($tid,$props);
		new_run($tid,'init');
	} elsif ( $lock =~ /at AR(9?)\/(\d+) on AR\1\/\2 failed/ ) {
		my $neuf = $1;
		my $oid = $2;
		info "La tache est bloquee sur un echec d'attente d'AR TD/9T. On suppose que ".
		     "les modifications necessaires ont ete apportees sur la demande de ".
		     "deploiement. Il faut purger l'attente d'AR, purger la demande TD/9T, puis ".
		     "relancer la demande TD/9T.";
		my $props = get_task_properties($tid);

		my $arkey = "AR$neuf/".$oid;
		remove_failed_subtask($tid,$props,$arkey);

		my $cdkey = $props->{'cde-key'};
		remove_done_subtask($tid,$props,$cdkey);
		
		update_task_properties($tid,$props);

		my $radkey = $props->{'radius-key'};
		new_run($tid,'FEOT-'.$radkey);
	} elsif ( $lock =~ /at CR(9?)\/(\d+) on (CR\1\/\2 failed|noevent)/) {
		my $neuf = $1;
		my $oid = $2;
		my $props = get_task_properties($tid);

		my $crkey = "CR$neuf/".$oid;
		remove_failed_subtask($tid,$props,$crkey);

		my $arkey = "AR$neuf/".$oid;
		remove_done_subtask($tid,$props,$arkey);

		my $cdkey = $props->{'cde-key'};
		remove_done_subtask($tid,$props,$cdkey);

		update_task_properties($tid,$props);

		my $radkey = $props->{'radius-key'};
		new_run($tid,'FEOT-'.$radkey);
	} else {
		error "La tache est bloquee dans ce cas: '$lock'. Il ne fait pas parti ".
		      "des cas connus.";
	}
}

my @tasks = get_locked_tasks('depl-adsl');
push @tasks, get_locked_tasks('depl-adsl9');
push @tasks, get_locked_tasks('depl-adslN');
push @tasks, get_locked_tasks('change-adsl9');
push @tasks, get_locked_tasks('change-adslN');
$page = tableau_2d('adsl/rewrite.tbl','tasklock',\@tasks).$page;
eject_page($page);

