#!/usr/bin/perl

use strict;
use CGI;
use FDN::Web::Forms;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;

my $cgi = new CGI;
my $vars = $cgi->Vars;

my $cfg = 'adsl/subs/souscription.cfg';
my $tbl = 'adsl/subs/souscription.tbl';

if ( !defined($vars->{'etape'}) ) {
	choixtarif();
} elsif ( $vars->{'etape'} eq 'choixtarif' ) {
	offre();
} elsif ( $vars->{'etape'} eq 'offre' ) {
	identite();
} elsif ( $vars->{'etape'} eq 'identite' ) {
	commande();
} elsif ( $vars->{'etape'} eq 'recap' ) {
	valide_commande();
} else {
	choixtarif();
}

sub choixtarif() {
	my $form = form_read ( $cfg, 'choixtarif' );
	eject_page(affiche_form($form,$vars));
}

sub fake_form_offre() {
	my $form = form_read ( $cfg, 'offre' );
	ajoute_champ_config($form,$cfg,'hoffre','hdebit');
	return $form;
}

sub make_form_offre($) {
	my ( $rej ) = @_;
	conduse('FDN::Adsl::Basic');
	conduse('FDN::Adsl::Souscription');
#	conduse('FDN::Adsl::Sel9');
	conduse('FDN::Adsl::SelN');
	my $tarifs = $FDN::Adsl::Souscription::tarifs;
	my @offres = @$FDN::Adsl::Souscription::offres;
	my $form = form_read ( $cfg, 'offre' );
	my $elig = eligibilite($vars->{tel},$vars->{cp},\@offres);

	if ( $elig->{etat} == 1 ) {
		my $page = "<p>Une erreur technique sur le serveur d'ligibilit ".
			   "nous met dans ".
			   "l'impossibilit de tester l'ligilit de votre ligne. Merci de ".
			   "ressayer plus tard.</p>\n".
			   "<p>Le texte des messages d'erreur:</p>\n";
		for my $m ( @{$elig->{'message'}} ) {
			$page .= "<p>".$m."</p>\n";
		}
		eject_page_error($page);
		exit(0);
	}
	if ( $elig->{etat} == 2 ) {
		eject_page_error("Une erreur technique chez FDN empche de poursuivre votre inscription");
		exit(0);
	}
	if ( $elig->{etat} == 3 ) {
		my $page = "<p>Il y a un problme avec le numro de tlphone que vous avez saisi.</p>\n";
		for my $m ( @{$elig->{'message'}} ) {
			$page .= "<p>".$m."</p>\n";
		}
		eject_page_error($page);
		exit(0);
	}
	if ( $elig->{etat} ) {
		my $page = "<p>Une erreur technique GRAVE chez FDN empche de traiter votre demande. Merci de ".
			   "transmettre l'information a FDN.</p>";
		eject_page_error($page);
		exit(0);
	}
	my $rejets;
	my $debits = 0;
	my $infoligne;
	for my $d ( @$FDN::Adsl::Souscription::offres ) {
		my $option = $d;
		my $debit = $d;
		if ( ! defined $elig ||
		     ! defined $elig->{'offres'} ||
		     ! defined $elig->{'offres'}->{$d} ||
		     ! $elig->{'offres'}->{$d}->{'faisable'} ) {
			debug "Debit $d pas faisable";
			conduse('FDN::Web::TemplateParse');
			$rejets .= remplace('/usr/share/fdn/templates/adsl/html/rejet-elig.tpl',$elig->{'offres'}->{$d},1);
			next;
		}

		if ( ($d eq '512kopt5') or ($d eq '512REopt5') ) {
			my $info = $elig->{'offres'}->{$d};
			$infoligne->{'length'} = $info->{'LineLength'};
			$infoligne->{'attenuation'} = $info->{'Attenuation'};
		}
		ajoute_champ_config($form,$cfg,'offre',$d);
		$debits++;
		$debit =~ s/opt\d$//;
		$option =~ s/^\d+(k|M|RE)//;
		$option = $FDN::Adsl::Souscription::options->{$option};
		my $did = debit_id_par_debit($debit);
		debug "Debit: $debit, donne $did; option: $option; tarif: ".$vars->{tarif};
		my $prid = prix_id_par_debit_id_type_option($did,$vars->{tarif},$option);
		debug "Prix id: $prid";
		my $pr = prix_par_id($prid);
		use Data::Dumper;
		debug "Prix: ".Dumper($pr);
		my $fas;
		my $abo;
		if ( $vars->{tarif} ne 'entreprise' ) {
			$fas = $pr->{PRIX_FASTTC}.' TTC';
			$abo = $pr->{PRIX_ABOTTC}.' TTC';
		} else {
			$fas = $pr->{PRIX_FASHT}.' HT';
			$abo = $pr->{PRIX_ABOHT}.' HT';
		}
		$fas =~ s/(\d{2}) /.$1&euro; /;
		$abo =~ s/(\d{2}) /.$1&euro; /;
		$form->{byname}->{$d}->{'input_label'}.=" $fas puis $abo par mois";
	}
	if (defined($infoligne->{'length'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> longueur : ". $infoligne->{'length'}. "m";
	}
	if (defined($infoligne->{'attenuation'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> attnuation thorique : ". $infoligne->{'attenuation'}. "dB";
	}
	if ( ! $debits ) {
		eject_page_error("<p>Il semble que vous ne soyiez pas ligible  un raccordement ADSL. Voil ".
				 "l'ensemble des informations que nous avons obtenu  propos de votre ligne.</p>".
				 $rejets);
		exit(0);
	}
	if ( defined $rej ) {
		$$rej = $rejets;
	}
	return $form;
}

sub offre() {
	my $form = form_read ( $cfg, 'choixtarif' );
	if ( !check_transition($form,$vars) ) {
		return choixtarif();
	}
	my $rejets;
	$form = make_form_offre(\$rejets);
	my $page = affiche_form($form,$vars);
	$page .= "<div class=\"rejets\"><p>Il y a certains dbit pour lesquels vous n'tes pas ligibles. Pour votre information, voil les raisons qui nous ont t donnes. Si certaines vous semblent fausses, n'hsitez pas  nous contacter.</p>$rejets</div>";
	eject_page($page);
}

sub identite() {
	my $form = fake_form_offre();
	if ( !check_transition($form,$vars) ) {
		return offre();
	}
	$form = form_read ( $cfg, 'identite' );
	eject_page(affiche_form($form,$vars));
}

sub commande() {
	my $form = form_read ( $cfg, 'identite' );
	if ( !check_transition($form,$vars) ) {
		eject_page(affiche_form($form,$vars));
		return;
	}
	if ( $vars->{banque} && $vars->{guichet} && $vars->{compte} && $vars->{rib} ) {
		my $ok = verif_clef_rib(
			$vars->{banque},
			$vars->{guichet},
			$vars->{compte},
			$vars->{rib});
		if ( ! $ok ) {
			add_form_error('syntax',
				"Il y a une erreur dans les coordonnees bancaires.");
			eject_page(affiche_form($form,$vars));
			return;
		}
	}
	conduse('FDN::Web::Array');
	$form = form_read ( $cfg, 'recap' );
	my $page = tableau_1d($tbl,'recap',$vars);
	$page .= affiche_form($form,$vars);
	eject_page($page);
}

sub valide_commande() {
	conduse('FDN::Adsl::Souscription');
	if ( ! $vars->{adressefac} ) {
		$vars->{adressefac} = $vars->{adressedflt};
	}
	if ( ! $vars->{cpfac} ) {
		$vars->{cpfac} = $vars->{cpdflt};
	}
	if ( ! $vars->{villefac} ) {
		$vars->{villefac} = $vars->{villedflt};
	}
	if ( ! $vars->{nomlig} ) {
		$vars->{nomlig} = $vars->{nomdflt};
	}

	my $dplnum = souscription_fdn($vars);
	if ( $dplnum ) {
		print $cgi->redirect('suivi-commande.cgi?dplid='.$dplnum.'&tel='.$vars->{tel}.'&do=yes');
	} else {
		my $page;
		$page = "<div class=\"error\"><p>Une erreur empeche de prendre vote commande\n".
			"en compte. Si cette erreur persiste, elle est probablement serieuse.\n".
			"Quoi qu'il en soit, les informations utiles ont ete transmises au\n".
			"bureau de l'association qui fera tout son possible pour que tout\n".
			"rentre dans l'ordre et vous re-contactera en cas de besoin.</p></div>";
		eject_page($page);
	}
}


